@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquicklabsplatformmenu_p.h"
 #include "qquicklabsplatformmenubar_p.h"
@@ -37,7 +38,7 @@ QT_BEGIN_NAMESPACE
 
     The Menu type provides a QML API for native platform menu popups.
 
-    \image qtlabsplatform-menu.png
+    \image {qtlabsplatform-menu.png} {A native popup menu}
 
     Menu can be used in a \l MenuBar, or as a stand-alone context menu.
     The following example shows how to open a context menu on right mouse
