@@ -1,6 +1,8 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
+#undef QT_NO_FOREACH // this file contains unported legacy Q_FOREACH uses
+
 #include <QtQuick/private/qsgcontext_p.h>
 #include <private/qsgadaptationlayer_p.h>
 #include <private/qquickitem_p.h>
@@ -17,7 +19,7 @@
 #include <private/qqmlglobal_p.h>
 #include <QtQml/qqmlinfo.h>
 #include <QtCore/QtMath>
-#include <QtGui/private/qrhi_p.h>
+#include <rhi/qrhi.h>
 
 #include <cmath>
 
@@ -52,41 +54,45 @@ class ImageMaterialData
 class TabledMaterialRhiShader : public QSGMaterialShader
 {
 public:
-    TabledMaterialRhiShader()
+    TabledMaterialRhiShader(int viewCount)
     {
-        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_tabled.vert.qsb"));
-        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_tabled.frag.qsb"));
+        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_tabled.vert.qsb"), viewCount);
+        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_tabled.frag.qsb"), viewCount);
     }
 
     bool updateUniformData(RenderState &renderState, QSGMaterial *newMaterial, QSGMaterial *) override
     {
         QByteArray *buf = renderState.uniformData();
         Q_ASSERT(buf->size() >= 80 + 2 * (UNIFORM_ARRAY_SIZE * 4 * 4));
+        const int shaderMatrixCount = newMaterial->viewCount();
+        const int matrixCount = qMin(renderState.projectionMatrixCount(), shaderMatrixCount);
 
-        if (renderState.isMatrixDirty()) {
-            const QMatrix4x4 m = renderState.combinedMatrix();
-            memcpy(buf->data(), m.constData(), 64);
+        for (int viewIndex = 0; viewIndex < matrixCount; ++viewIndex) {
+            if (renderState.isMatrixDirty()) {
+                const QMatrix4x4 m = renderState.combinedMatrix(viewIndex);
+                memcpy(buf->data() + 64 * viewIndex, m.constData(), 64);
+            }
         }
 
         if (renderState.isOpacityDirty()) {
             const float opacity = renderState.opacity();
-            memcpy(buf->data() + 64, &opacity, 4);
+            memcpy(buf->data() + 64 * shaderMatrixCount, &opacity, 4);
         }
 
         ImageMaterialData *state = static_cast<ImageMaterial *>(newMaterial)->state();
 
         float entry = float(state->entry);
-        memcpy(buf->data() + 68, &entry, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 4, &entry, 4);
 
         float timestamp = float(state->timestamp);
-        memcpy(buf->data() + 72, &timestamp, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 8, &timestamp, 4);
 
-        float *p = reinterpret_cast<float *>(buf->data() + 80);
+        float *p = reinterpret_cast<float *>(buf->data() + 64 * shaderMatrixCount + 16);
         for (int i = 0; i < UNIFORM_ARRAY_SIZE; ++i) {
             *p = state->sizeTable[i];
             p += 4;
         }
-        p = reinterpret_cast<float *>(buf->data() + 80 + (UNIFORM_ARRAY_SIZE * 4 * 4));
+        p = reinterpret_cast<float *>(buf->data() + 64 * shaderMatrixCount + 16 + (UNIFORM_ARRAY_SIZE * 4 * 4));
         for (int i = 0; i < UNIFORM_ARRAY_SIZE; ++i) {
             *p = state->opacityTable[i];
             p += 4;
@@ -114,7 +120,7 @@ class TabledMaterial : public ImageMaterial
 public:
     QSGMaterialShader *createShader(QSGRendererInterface::RenderMode renderMode) const override {
         Q_UNUSED(renderMode);
-        return new TabledMaterialRhiShader;
+        return new TabledMaterialRhiShader(viewCount());
     }
     QSGMaterialType *type() const override { return &m_type; }
 
@@ -130,34 +136,38 @@ QSGMaterialType TabledMaterial::m_type;
 class DeformableMaterialRhiShader : public QSGMaterialShader
 {
 public:
-    DeformableMaterialRhiShader()
+    DeformableMaterialRhiShader(int viewCount)
     {
-        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_deformed.vert.qsb"));
-        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_deformed.frag.qsb"));
+        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_deformed.vert.qsb"), viewCount);
+        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_deformed.frag.qsb"), viewCount);
     }
 
     bool updateUniformData(RenderState &renderState, QSGMaterial *newMaterial, QSGMaterial *) override
     {
         QByteArray *buf = renderState.uniformData();
         Q_ASSERT(buf->size() >= 80 + 2 * (UNIFORM_ARRAY_SIZE * 4 * 4));
+        const int shaderMatrixCount = newMaterial->viewCount();
+        const int matrixCount = qMin(renderState.projectionMatrixCount(), shaderMatrixCount);
 
-        if (renderState.isMatrixDirty()) {
-            const QMatrix4x4 m = renderState.combinedMatrix();
-            memcpy(buf->data(), m.constData(), 64);
+        for (int viewIndex = 0; viewIndex < matrixCount; ++viewIndex) {
+            if (renderState.isMatrixDirty()) {
+                const QMatrix4x4 m = renderState.combinedMatrix(viewIndex);
+                memcpy(buf->data() + 64 * viewIndex, m.constData(), 64);
+            }
         }
 
         if (renderState.isOpacityDirty()) {
             const float opacity = renderState.opacity();
-            memcpy(buf->data() + 64, &opacity, 4);
+            memcpy(buf->data() + 64 * shaderMatrixCount, &opacity, 4);
         }
 
         ImageMaterialData *state = static_cast<ImageMaterial *>(newMaterial)->state();
 
         float entry = float(state->entry);
-        memcpy(buf->data() + 68, &entry, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 4, &entry, 4);
 
         float timestamp = float(state->timestamp);
-        memcpy(buf->data() + 72, &timestamp, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 8, &timestamp, 4);
 
         return true;
     }
@@ -178,7 +188,7 @@ class DeformableMaterial : public ImageMaterial
 public:
     QSGMaterialShader *createShader(QSGRendererInterface::RenderMode renderMode) const override {
         Q_UNUSED(renderMode);
-        return new DeformableMaterialRhiShader;
+        return new DeformableMaterialRhiShader(viewCount());
     }
     QSGMaterialType *type() const override { return &m_type; }
 
@@ -194,41 +204,45 @@ QSGMaterialType DeformableMaterial::m_type;
 class ParticleSpriteMaterialRhiShader : public QSGMaterialShader
 {
 public:
-    ParticleSpriteMaterialRhiShader()
+    ParticleSpriteMaterialRhiShader(int viewCount)
     {
-        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_sprite.vert.qsb"));
-        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_sprite.frag.qsb"));
+        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_sprite.vert.qsb"), viewCount);
+        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_sprite.frag.qsb"), viewCount);
     }
 
     bool updateUniformData(RenderState &renderState, QSGMaterial *newMaterial, QSGMaterial *) override
     {
         QByteArray *buf = renderState.uniformData();
         Q_ASSERT(buf->size() >= 80 + 2 * (UNIFORM_ARRAY_SIZE * 4 * 4));
+        const int shaderMatrixCount = newMaterial->viewCount();
+        const int matrixCount = qMin(renderState.projectionMatrixCount(), shaderMatrixCount);
 
-        if (renderState.isMatrixDirty()) {
-            const QMatrix4x4 m = renderState.combinedMatrix();
-            memcpy(buf->data(), m.constData(), 64);
+        for (int viewIndex = 0; viewIndex < matrixCount; ++viewIndex) {
+            if (renderState.isMatrixDirty()) {
+                const QMatrix4x4 m = renderState.combinedMatrix(viewIndex);
+                memcpy(buf->data() + 64 * viewIndex, m.constData(), 64);
+            }
         }
 
         if (renderState.isOpacityDirty()) {
             const float opacity = renderState.opacity();
-            memcpy(buf->data() + 64, &opacity, 4);
+            memcpy(buf->data() + 64 * shaderMatrixCount, &opacity, 4);
         }
 
         ImageMaterialData *state = static_cast<ImageMaterial *>(newMaterial)->state();
 
         float entry = float(state->entry);
-        memcpy(buf->data() + 68, &entry, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 4, &entry, 4);
 
         float timestamp = float(state->timestamp);
-        memcpy(buf->data() + 72, &timestamp, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 8, &timestamp, 4);
 
-        float *p = reinterpret_cast<float *>(buf->data() + 80);
+        float *p = reinterpret_cast<float *>(buf->data() + 64 * shaderMatrixCount + 16);
         for (int i = 0; i < UNIFORM_ARRAY_SIZE; ++i) {
             *p = state->sizeTable[i];
             p += 4;
         }
-        p = reinterpret_cast<float *>(buf->data() + 80 + (UNIFORM_ARRAY_SIZE * 4 * 4));
+        p = reinterpret_cast<float *>(buf->data() + 64 * shaderMatrixCount + 16 + (UNIFORM_ARRAY_SIZE * 4 * 4));
         for (int i = 0; i < UNIFORM_ARRAY_SIZE; ++i) {
             *p = state->opacityTable[i];
             p += 4;
@@ -256,7 +270,7 @@ class SpriteMaterial : public ImageMaterial
 public:
     QSGMaterialShader *createShader(QSGRendererInterface::RenderMode renderMode) const override {
         Q_UNUSED(renderMode);
-        return new ParticleSpriteMaterialRhiShader;
+        return new ParticleSpriteMaterialRhiShader(viewCount());
     }
     QSGMaterialType *type() const override { return &m_type; }
 
@@ -272,37 +286,41 @@ QSGMaterialType SpriteMaterial::m_type;
 class ColoredPointMaterialRhiShader : public QSGMaterialShader
 {
 public:
-    ColoredPointMaterialRhiShader()
+    ColoredPointMaterialRhiShader(int viewCount)
     {
-        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_coloredpoint.vert.qsb"));
-        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_coloredpoint.frag.qsb"));
+        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_coloredpoint.vert.qsb"), viewCount);
+        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_coloredpoint.frag.qsb"), viewCount);
     }
 
     bool updateUniformData(RenderState &renderState, QSGMaterial *newMaterial, QSGMaterial *) override
     {
         QByteArray *buf = renderState.uniformData();
         Q_ASSERT(buf->size() >= 80 + 2 * (UNIFORM_ARRAY_SIZE * 4 * 4));
+        const int shaderMatrixCount = newMaterial->viewCount();
+        const int matrixCount = qMin(renderState.projectionMatrixCount(), shaderMatrixCount);
 
-        if (renderState.isMatrixDirty()) {
-            const QMatrix4x4 m = renderState.combinedMatrix();
-            memcpy(buf->data(), m.constData(), 64);
+        for (int viewIndex = 0; viewIndex < matrixCount; ++viewIndex) {
+            if (renderState.isMatrixDirty()) {
+                const QMatrix4x4 m = renderState.combinedMatrix(viewIndex);
+                memcpy(buf->data() + 64 * viewIndex, m.constData(), 64);
+            }
         }
 
         if (renderState.isOpacityDirty()) {
             const float opacity = renderState.opacity();
-            memcpy(buf->data() + 64, &opacity, 4);
+            memcpy(buf->data() + 64 * shaderMatrixCount, &opacity, 4);
         }
 
         ImageMaterialData *state = static_cast<ImageMaterial *>(newMaterial)->state();
 
         float entry = float(state->entry);
-        memcpy(buf->data() + 68, &entry, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 4, &entry, 4);
 
         float timestamp = float(state->timestamp);
-        memcpy(buf->data() + 72, &timestamp, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 8, &timestamp, 4);
 
         float dpr = float(state->dpr);
-        memcpy(buf->data() + 76, &dpr, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 12, &dpr, 4);
 
         return true;
     }
@@ -323,7 +341,7 @@ class ColoredPointMaterial : public ImageMaterial
 public:
     QSGMaterialShader *createShader(QSGRendererInterface::RenderMode renderMode) const override {
         Q_UNUSED(renderMode);
-        return new ColoredPointMaterialRhiShader;
+        return new ColoredPointMaterialRhiShader(viewCount());
     }
     QSGMaterialType *type() const override { return &m_type; }
 
@@ -339,10 +357,11 @@ QSGMaterialType ColoredPointMaterial::m_type;
 class ColoredMaterialRhiShader : public ColoredPointMaterialRhiShader
 {
 public:
-    ColoredMaterialRhiShader()
+    ColoredMaterialRhiShader(int viewCount)
+        : ColoredPointMaterialRhiShader(viewCount)
     {
-        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_colored.vert.qsb"));
-        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_colored.frag.qsb"));
+        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_colored.vert.qsb"), viewCount);
+        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_colored.frag.qsb"), viewCount);
     }
 };
 
@@ -351,7 +370,7 @@ class ColoredMaterial : public ImageMaterial
 public:
     QSGMaterialShader *createShader(QSGRendererInterface::RenderMode renderMode) const override {
         Q_UNUSED(renderMode);
-        return new ColoredMaterialRhiShader;
+        return new ColoredMaterialRhiShader(viewCount());
     }
     QSGMaterialType *type() const override { return &m_type; }
 
@@ -367,37 +386,41 @@ QSGMaterialType ColoredMaterial::m_type;
 class SimplePointMaterialRhiShader : public QSGMaterialShader
 {
 public:
-    SimplePointMaterialRhiShader()
+    SimplePointMaterialRhiShader(int viewCount)
     {
-        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_simplepoint.vert.qsb"));
-        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_simplepoint.frag.qsb"));
+        setShaderFileName(VertexStage, QStringLiteral(":/particles/shaders_ng/imageparticle_simplepoint.vert.qsb"), viewCount);
+        setShaderFileName(FragmentStage, QStringLiteral(":/particles/shaders_ng/imageparticle_simplepoint.frag.qsb"), viewCount);
     }
 
     bool updateUniformData(RenderState &renderState, QSGMaterial *newMaterial, QSGMaterial *) override
     {
         QByteArray *buf = renderState.uniformData();
         Q_ASSERT(buf->size() >= 80 + 2 * (UNIFORM_ARRAY_SIZE * 4 * 4));
+        const int shaderMatrixCount = newMaterial->viewCount();
+        const int matrixCount = qMin(renderState.projectionMatrixCount(), shaderMatrixCount);
 
-        if (renderState.isMatrixDirty()) {
-            const QMatrix4x4 m = renderState.combinedMatrix();
-            memcpy(buf->data(), m.constData(), 64);
+        for (int viewIndex = 0; viewIndex < matrixCount; ++viewIndex) {
+            if (renderState.isMatrixDirty()) {
+                const QMatrix4x4 m = renderState.combinedMatrix(viewIndex);
+                memcpy(buf->data() + 64 * viewIndex, m.constData(), 64);
+            }
         }
 
         if (renderState.isOpacityDirty()) {
             const float opacity = renderState.opacity();
-            memcpy(buf->data() + 64, &opacity, 4);
+            memcpy(buf->data() + 64 * shaderMatrixCount, &opacity, 4);
         }
 
         ImageMaterialData *state = static_cast<ImageMaterial *>(newMaterial)->state();
 
         float entry = float(state->entry);
-        memcpy(buf->data() + 68, &entry, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 4, &entry, 4);
 
         float timestamp = float(state->timestamp);
-        memcpy(buf->data() + 72, &timestamp, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 8, &timestamp, 4);
 
         float dpr = float(state->dpr);
-        memcpy(buf->data() + 76, &dpr, 4);
+        memcpy(buf->data() + 64 * shaderMatrixCount + 12, &dpr, 4);
 
         return true;
     }
@@ -418,7 +441,7 @@ class SimplePointMaterial : public ImageMaterial
 public:
     QSGMaterialShader *createShader(QSGRendererInterface::RenderMode renderMode) const override {
         Q_UNUSED(renderMode);
-        return new SimplePointMaterialRhiShader;
+        return new SimplePointMaterialRhiShader(viewCount());
     }
     QSGMaterialType *type() const override { return &m_type; }
 
@@ -445,7 +468,7 @@ void fillUniformArrayFromImage(float* array, const QImage& img, int size)
 
 /*!
     \qmltype ImageParticle
-    \instantiates QQuickImageParticle
+    \nativetype QQuickImageParticle
     \inqmlmodule QtQuick.Particles
     \inherits ParticlePainter
     \brief For visualizing logical particles using an image.
@@ -651,13 +674,12 @@ void fillUniformArrayFromImage(float* array, const QImage& img, int size)
     For fine-grained control, see sizeTable and opacityTable.
 
     Acceptable values are
-    \list
-    \li ImageParticle.None: Particles just appear and disappear.
-    \li ImageParticle.Fade: Particles fade in from 0 opacity at the start of their life, and fade out to 0 at the end.
-    \li ImageParticle.Scale: Particles scale in from 0 size at the start of their life, and scale back to 0 at the end.
-    \endlist
 
-    Default value is Fade.
+    \value ImageParticle.None   Particles just appear and disappear.
+    \value ImageParticle.Fade   Particles fade in from 0 opacity at the start of their life, and fade out to 0 at the end.
+    \value ImageParticle.Scale  Particles scale in from 0 size at the start of their life, and scale back to 0 at the end.
+
+    The default value is \c ImageParticle.Fade.
 */
 /*!
     \qmlproperty bool QtQuick.Particles::ImageParticle::spritesInterpolate
@@ -730,6 +752,7 @@ void QQuickImageParticle::sceneGraphInvalidated()
     m_material = nullptr;
     delete m_outgoingNode;
     m_outgoingNode = nullptr;
+    m_apiChecked = false;
 }
 
 void QQuickImageParticle::setImage(const QUrl &image)
@@ -1038,14 +1061,20 @@ void QQuickImageParticle::reset()
     update();
 }
 
+
+void QQuickImageParticle::invalidateSceneGraph()
+{
+    reset();
+}
+
 void QQuickImageParticle::createEngine()
 {
     if (m_spriteEngine)
         delete m_spriteEngine;
     if (m_sprites.size()) {
         m_spriteEngine = new QQuickSpriteEngine(m_sprites, this);
-        connect(m_spriteEngine, SIGNAL(stateChanged(int)),
-                this, SLOT(spriteAdvance(int)), Qt::DirectConnection);
+        connect(m_spriteEngine, &QQuickStochasticEngine::stateChanged,
+                this, &QQuickImageParticle::spriteAdvance, Qt::DirectConnection);
         m_explicitAnimation = true;
     } else {
         m_spriteEngine = nullptr;
@@ -1408,7 +1437,7 @@ void QQuickImageParticle::finishBuildParticleNodes(QSGNode** node)
 
         m_nodes.insert(groupId, node);
         m_idxStarts.insert(groupId, m_lastIdxStart);
-        m_startsIdx.append(qMakePair(m_lastIdxStart, groupId));
+        m_startsIdx.append(std::make_pair(m_lastIdxStart, groupId));
         m_lastIdxStart += count;
 
         //Create Particle Geometry
@@ -1827,6 +1856,7 @@ void QQuickImageParticle::initialize(int gIdx, int pIdx)
                 else
                     getShadowDatum(datum)->color = color;
             }
+            break;
         default:
             break;
     }
