@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant
 #ifndef QV4COMPILEDDATA_P_H
 #define QV4COMPILEDDATA_P_H
 
@@ -51,7 +52,7 @@ QT_BEGIN_NAMESPACE
 // Also change the comment behind the number to describe the latest change. This has the added
 // benefit that if another patch changes the version too, it will result in a merge conflict, and
 // not get removed silently.
-#define QV4_DATA_STRUCTURE_VERSION 0x42 // Change metatype computation of AOT-compiled functions
+#define QV4_DATA_STRUCTURE_VERSION 0x48 // Hotfix 6.10 - meta object change
 
 class QIODevice;
 class QQmlTypeNameCache;
@@ -156,7 +157,7 @@ static_assert(sizeof(Location) == 4, "Location structure needs to have the expec
 
 struct RegExp
 {
-    enum Flags : unsigned int {
+    enum Flag : quint8 {
         RegExp_NoFlags    = 0x0,
         RegExp_Global     = 0x01,
         RegExp_IgnoreCase = 0x02,
@@ -165,6 +166,8 @@ struct RegExp
         RegExp_Unicode    = 0x10,
     };
 
+    Q_DECLARE_FLAGS(Flags, Flag);
+
     RegExp() : m_data(QSpecialIntegerBitfieldZero) {}
     RegExp(quint32 flags, quint32 stringIndex) : RegExp()
     {
@@ -788,14 +791,16 @@ static_assert(sizeof(Signal) == 12, "Signal structure needs to have the expected
 struct Property
 {
 private:
+    using NameIndexField = quint32_le_bitfield_member<0, 31>;
+    using FinalField = quint32_le_bitfield_member<31, 1>;
+
     using CommonTypeOrTypeNameIndexField = quint32_le_bitfield_member<0, 28>;
     using IsRequiredField = quint32_le_bitfield_member<28, 1>;
     using IsCommonTypeField = quint32_le_bitfield_member<29, 1>;
     using IsListField = quint32_le_bitfield_member<30, 1>;
     using IsReadOnlyField = quint32_le_bitfield_member<31, 1>;
-
 public:
-    quint32_le nameIndex;
+    quint32_le_bitfield_union<NameIndexField, FinalField> nameIndexAndFinal;
     quint32_le_bitfield_union<
             CommonTypeOrTypeNameIndexField,
             IsRequiredField,
@@ -804,6 +809,12 @@ public:
             IsReadOnlyField> data;
     Location location;
 
+    quint32 nameIndex() const { return nameIndexAndFinal.get<NameIndexField>(); }
+    void setNameIndex(int nameIndex) { nameIndexAndFinal.set<NameIndexField>(nameIndex); }
+
+    bool isFinal() const { return nameIndexAndFinal.get<FinalField>(); }
+    void setIsFinal(bool final) { nameIndexAndFinal.set<FinalField>(final ? 1 : 0); }
+
     void setCommonType(CommonType t)
     {
         data.set<CommonTypeOrTypeNameIndexField>(static_cast<quint32>(t));
@@ -861,8 +872,8 @@ public:
 
     enum Flag : unsigned int {
         IsReadOnly = 0x1,
-        Resolved = 0x2,
-        AliasPointsToPointerObject = 0x4
+        AliasPointsToPointerObject = 0x2,
+        // One bit vacant
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
@@ -872,7 +883,6 @@ public:
 
     union {
         quint32_le propertyNameIndex; // string index
-        qint32_le encodedMetaPropertyIndex;
         quint32_le localAliasIndex; // index in list of aliases local to the object (if targetObjectId == objectId)
     };
     Location location;
@@ -903,12 +913,6 @@ public:
         nameIndexAndFlags.set<NameIndexField>(nameIndex);
     }
 
-    bool isObjectAlias() const
-    {
-        Q_ASSERT(hasFlag(Resolved));
-        return encodedMetaPropertyIndex == -1;
-    }
-
     quint32 idIndex() const
     {
         return idIndexAndTargetObjectIdAndAliasToLocalAlias.get<IdIndexField>();
@@ -1193,7 +1197,6 @@ struct QmlUnit
 };
 static_assert(sizeof(QmlUnit) == 16, "QmlUnit structure needs to have the expected size to be binary compatible on disk when generated by host compiler and loaded by target");
 
-enum { QmlCompileHashSpace = 48 };
 static const char magic_str[] = "qv4cdata";
 
 struct Unit
@@ -1201,12 +1204,11 @@ struct Unit
     // DO NOT CHANGE THESE FIELDS EVER
     char magic[8];
     quint32_le version;
-    quint32_le qtVersion;
-    qint64_le sourceTimeStamp;
-    quint32_le unitSize; // Size of the Unit and any depending data.
     // END DO NOT CHANGE THESE FIELDS EVER
 
-    char libraryVersionHash[QmlCompileHashSpace];
+    quint32_le reserved; // For predictable alignment and size. Used to be Qt version.
+    qint64_le sourceTimeStamp;
+    quint32_le unitSize; // Size of the Unit and any depending data.
 
     char md5Checksum[16]; // checksum of all bytes following this field.
     char dependencyMD5Checksum[16];
@@ -1377,7 +1379,7 @@ struct Unit
     bool verifyHeader(QDateTime expectedSourceTimeStamp, QString *errorString) const;
 };
 
-static_assert(sizeof(Unit) == 248, "Unit structure needs to have the expected size to be binary compatible on disk when generated by host compiler and loaded by target");
+static_assert(sizeof(Unit) == 200, "Unit structure needs to have the expected size to be binary compatible on disk when generated by host compiler and loaded by target");
 
 struct TypeReference
 {
@@ -1446,23 +1448,15 @@ using DependentTypesHasher = std::function<QByteArray()>;
 struct InlineComponentData {
 
     InlineComponentData() = default;
-    InlineComponentData(
-            const QQmlType &qmlType, int objectIndex, int nameIndex, int totalObjectCount,
-            int totalBindingCount, int totalParserStatusCount)
+    InlineComponentData(const QQmlType &qmlType, int objectIndex, int nameIndex)
         : qmlType(qmlType)
         , objectIndex(objectIndex)
         , nameIndex(nameIndex)
-        , totalObjectCount(totalObjectCount)
-        , totalBindingCount(totalBindingCount)
-        , totalParserStatusCount(totalParserStatusCount)
     {}
 
     QQmlType qmlType;
     int objectIndex = -1;
     int nameIndex = -1;
-    int totalObjectCount = 0;
-    int totalBindingCount = 0;
-    int totalParserStatusCount = 0;
 };
 
 struct CompilationUnit final : public QQmlRefCounted<CompilationUnit>
@@ -1479,10 +1473,6 @@ struct CompilationUnit final : public QQmlRefCounted<CompilationUnit>
 
     std::unique_ptr<CompilationUnitMapper> backingFile;
 
-    int m_totalBindingsCount = 0; // Number of bindings used in this type
-    int m_totalParserStatusCount = 0; // Number of instantiated types that are QQmlParserStatus subclasses
-    int m_totalObjectCount = 0; // Number of objects explicitly instantiated
-
     QHash<QString, InlineComponentData> inlineComponentData;
 
     // index is object index. This allows fast access to the
@@ -1656,10 +1646,6 @@ public:
     int objectCount() const { return qmlData->nObjects; }
     const CompiledObject *objectAt(int index) const { return qmlData->objectAt(index); }
 
-    int totalBindingsCount(const QString &inlineComponentRootName) const;
-    int totalParserStatusCount(const QString &inlineComponentRootName) const;
-    int totalObjectCount(const QString &inlineComponentRootName) const;
-
     int inlineComponentId(const QString &inlineComponentName) const
     {
         for (uint i = 0; i < qmlData->nObjects; ++i) {
