@@ -1,30 +1,32 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include "qv4engine_p.h"
-#include "qv4object_p.h"
-#include "qv4mm_p.h"
-#include "qv4qobjectwrapper_p.h"
-#include "qv4identifiertable_p.h"
-#include <QtCore/qalgorithms.h>
-#include <QtCore/private/qnumeric_p.h>
-#include <QtCore/qloggingcategory.h>
-#include <private/qv4alloca_p.h>
-#include <qqmlengine.h>
-#include "PageReservation.h"
 #include "PageAllocation.h"
+#include "PageReservation.h"
 
-#include <QElapsedTimer>
-#include <QMap>
-#include <QScopedValueRollback>
+#include <private/qnumeric_p.h>
+#include <private/qv4alloca_p.h>
+#include <private/qv4engine_p.h>
+#include <private/qv4identifiertable_p.h>
+#include <private/qv4mapobject_p.h>
+#include <private/qv4mm_p.h>
+#include <private/qv4object_p.h>
+#include <private/qv4profiling_p.h>
+#include <private/qv4qobjectwrapper_p.h>
+#include <private/qv4setobject_p.h>
+#include <private/qv4stackframe_p.h>
+
+#include <QtQml/qqmlengine.h>
+
+#include <QtCore/qalgorithms.h>
+#include <QtCore/qelapsedtimer.h>
+#include <QtCore/qloggingcategory.h>
+#include <QtCore/qmap.h>
+#include <QtCore/qscopedvaluerollback.h>
 
-#include <cstdlib>
 #include <algorithm>
-#include "qv4profiling_p.h"
-#include "qv4mapobject_p.h"
-#include "qv4setobject_p.h"
-
 #include <chrono>
+#include <cstdlib>
 
 //#define MM_STATS
 
@@ -552,6 +554,47 @@ done:
     return m;
 }
 
+// index based version of partition to handle potential growth at the end
+template<class UnaryPred>
+qsizetype partition(std::vector<Chunk *> &container, UnaryPred p)
+{
+    std::size_t first = 0, last = container.size();
+    while (first != last && p(container.at(first)))
+        ++first;
+    // handle the case where the container has grown
+    if (last != container.size()) {
+        while (first != container.size() && p(container.at(first)))
+            ++first;
+        last = container.size();
+    }
+
+    if (first == last)
+        return first;
+
+    for (auto i = first++; i != last; ++i) {
+        if (p(container.at(i))) {
+            std::swap(container.at(i), container.at(first));
+            ++first;
+        }
+    }
+
+    // handle the case that new chunks have been added
+    auto newSize = container.size();
+    if (last != newSize) {
+        Q_ASSERT(last < newSize);
+        // must check container.size() instead of newSize, as we would
+        // still grow the container
+        for (auto i = last; i != container.size(); ++i) {
+            if (p(container.at(i))) {
+                std::swap(container.at(i), container.at(first));
+                ++first;
+            }
+        }
+    }
+
+    return first;
+}
+
 void BlockAllocator::sweep()
 {
     nextFree = nullptr;
@@ -561,9 +604,10 @@ void BlockAllocator::sweep()
 //    qDebug() << "BlockAlloc: sweep";
     usedSlotsAfterLastSweep = 0;
 
-    auto firstEmptyChunk = std::partition(chunks.begin(), chunks.end(), [this](Chunk *c) {
+    auto firstEmptyChunkPos = partition(chunks, [this](Chunk *c) {
         return c->sweep(engine);
     });
+    auto firstEmptyChunk = chunks.begin() + firstEmptyChunkPos;
 
     std::for_each(chunks.begin(), firstEmptyChunk, [this](Chunk *c) {
         c->sortIntoBins(freeBins, NumBins);
@@ -1310,7 +1354,7 @@ void MemoryManager::runGC()
         t.start();
         gcStateMachine->step();
         qint64 markTime = t.nsecsElapsed()/1000;
-        t.restart();
+        t.start();
         const size_t usedAfter = getUsedMem();
         const size_t largeItemsAfter = getLargeItemsMem();
 
@@ -1473,6 +1517,19 @@ void MemoryManager::collectFromJSStack(MarkStack *markStack) const
         }
         ++v;
     }
+
+    for (auto *frame = engine->currentStackFrame; frame; frame = frame->parentFrame()) {
+        if (!frame->isMetaTypesFrame())
+            continue;
+
+        if (const QQmlPrivate::AOTTrackedLocalsStorage *locals
+                = static_cast<const MetaTypesStackFrame *>(frame)->locals()) {
+            // Actual AOT-compiled functions initialize the locals firsth thing when they
+            // are called. However, the ScopedStackFrame has no locals, but still uses a
+            // MetaTypesStackFrame.
+            locals->markObjects(markStack);
+        }
+    }
 }
 
 GCStateMachine::GCStateMachine()
