@@ -27,13 +27,13 @@ void QQmlQmldirData::setPriority(
         QQmlTypeLoader::Blob *blob, const QQmlTypeLoader::Blob::PendingImportPtr &import,
         int priority)
 {
-    Q_ASSERT(isTypeLoaderThread());
+    assertTypeLoaderThread();
     m_imports.insert(blob, { import, priority });
 }
 
 void QQmlQmldirData::dataReceived(const SourceCodeData &data)
 {
-    Q_ASSERT(isTypeLoaderThread());
+    assertTypeLoaderThread();
     QString error;
     m_content = data.readAll(&error);
     if (!error.isEmpty()) {
@@ -44,7 +44,7 @@ void QQmlQmldirData::dataReceived(const SourceCodeData &data)
 
 void QQmlQmldirData::initializeFromCachedUnit(const QQmlPrivate::CachedQmlUnit *)
 {
-    Q_ASSERT(isTypeLoaderThread());
+    assertTypeLoaderThread();
     Q_UNIMPLEMENTED();
 }
 
