@@ -43,17 +43,53 @@ void QQmlIRLoader::load()
         createPragma(type)->componentBehavior = value;
     };
 
+    const auto createFunctionSignaturePragma = [&](
+            Pragma::PragmaType type,
+            Pragma::FunctionSignatureBehaviorValue value) {
+        createPragma(type)->functionSignatureBehavior = value;
+    };
+
+    const auto createNativeMethodPragma = [&](
+            Pragma::PragmaType type,
+            Pragma::NativeMethodBehaviorValue value) {
+        createPragma(type)->nativeMethodBehavior = value;
+    };
+
+    const auto createValueTypePragma = [&](
+            Pragma::PragmaType type,
+            Pragma::ValueTypeBehaviorValues value) {
+        createPragma(type)->valueTypeBehavior = value;
+    };
+
     if (unit->flags & QV4::CompiledData::Unit::IsSingleton)
         createPragma(Pragma::Singleton);
     if (unit->flags & QV4::CompiledData::Unit::IsStrict)
         createPragma(Pragma::Strict);
+
     if (unit->flags & QV4::CompiledData::Unit::ListPropertyAssignReplace)
         createListPragma(Pragma::ListPropertyAssignBehavior, Pragma::Replace);
     else if (unit->flags & QV4::CompiledData::Unit::ListPropertyAssignReplaceIfNotDefault)
         createListPragma(Pragma::ListPropertyAssignBehavior, Pragma::ReplaceIfNotDefault);
+
     if (unit->flags & QV4::CompiledData::Unit::ComponentsBound)
         createComponentPragma(Pragma::ComponentBehavior, Pragma::Bound);
 
+    if (unit->flags & QV4::CompiledData::Unit::FunctionSignaturesIgnored)
+        createFunctionSignaturePragma(Pragma::FunctionSignatureBehavior, Pragma::Ignored);
+
+    if (unit->flags & QV4::CompiledData::Unit::NativeMethodsAcceptThisObject)
+        createNativeMethodPragma(Pragma::NativeMethodBehavior, Pragma::AcceptThisObject);
+
+    Pragma::ValueTypeBehaviorValues valueTypeBehavior = {};
+    if (unit->flags & QV4::CompiledData::Unit::ValueTypesCopied)
+        valueTypeBehavior |= Pragma::Copy;
+    if (unit->flags & QV4::CompiledData::Unit::ValueTypesAddressable)
+        valueTypeBehavior |= Pragma::Addressable;
+    if (unit->flags & QV4::CompiledData::Unit::ValueTypesAssertable)
+        valueTypeBehavior |= Pragma::Assertable;
+    if (valueTypeBehavior)
+        createValueTypePragma(Pragma::ValueTypeBehavior, valueTypeBehavior);
+
     for (uint i = 0; i < qmlUnit->nObjects; ++i) {
         const QV4::CompiledData::Object *serializedObject = qmlUnit->objectAt(i);
         QmlIR::Object *object = loadObject(serializedObject);
