@@ -57,7 +57,7 @@ QQmlRefPointer<QQmlContextData> QQmlScriptData::qmlContextDataForContext(
     }
     QV4::ScopedValue v(scope);
     for (int ii = 0; ii < scripts.size(); ++ii) {
-        v = scripts.at(ii)->scriptData()->scriptValueForContext(qmlContextData);
+        v = scripts.at(ii)->scriptValueForContext(qmlContextData);
         scriptsArray->put(ii, v);
     }
 
