@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// Qt-Security score:significant
 
 #include "qqmljstypereader_p.h"
 #include "qqmljsimportvisitor_p.h"
@@ -44,23 +45,22 @@ bool QQmlJSTypeReader::operator ()(const QSharedPointer<QQmlJSScope> &scope)
     const bool success = isJavaScript ? (isESModule ? parser.parseModule()
                                                     : parser.parseProgram())
                                       : parser.parse();
-    if (!success)
-        return false;
 
     QQmlJS::AST::Node *rootNode = parser.rootNode();
-    if (!rootNode)
-        return false;
 
     QQmlJSLogger logger;
-    logger.setFileName(m_file);
+    logger.setFilePath(m_file);
     logger.setCode(code);
     logger.setSilent(true);
+    logger.setIsDisabled(true);
 
-    auto membersVisitor = m_importer->makeImportVisitor(
-            scope, m_importer, &logger,
-            QQmlJSImportVisitor::implicitImportDirectory(m_file, m_importer->resourceFileMapper()));
-    rootNode->accept(membersVisitor.get());
-    return true;
+    m_importer->runImportVisitor(rootNode,
+                                 { scope,
+                                   &logger,
+                                   QQmlJSImportVisitor::implicitImportDirectory(
+                                           m_file, m_importer->resourceFileMapper()),
+                                   {} });
+    return success && rootNode;
 }
 
 QT_END_NAMESPACE
