@@ -380,76 +380,76 @@ const FieldFilter &DocumentSymbolVisitor::fieldsFilter()
         {}, // to add
         {
                 // to remove
-                { QString(), QString::fromUtf16(Fields::code) },
-                { QString(), QString::fromUtf16(Fields::postCode) },
-                { QString(), QString::fromUtf16(Fields::preCode) },
-                { QString(), QString::fromUtf16(Fields::importScope) },
-                { QString(), QString::fromUtf16(Fields::fileLocationsTree) },
-                { QString(), QString::fromUtf16(Fields::astComments) },
-                { QString(), QString::fromUtf16(Fields::comments) },
-                { QString(), QString::fromUtf16(Fields::exports) },
-                { QString(), QString::fromUtf16(Fields::propertyInfos) },
-                { QLatin1String("FileLocationsNode"), QString::fromUtf16(Fields::parent) },
+                { QString(), Fields::code.toString() },
+                { QString(), Fields::postCode.toString() },
+                { QString(), Fields::preCode.toString() },
+                { QString(), Fields::importScope.toString() },
+                { QString(), Fields::fileLocationsTree.toString() },
+                { QString(), Fields::astComments.toString() },
+                { QString(), Fields::comments.toString() },
+                { QString(), Fields::exports.toString() },
+                { QString(), Fields::propertyInfos.toString() },
+                { QLatin1String("FileLocationsNode"), Fields::parent.toString() },
                 //^^^ FieldFilter::default
-                { QString(), QString::fromUtf16(Fields::errors) },
-                { QString(), QString::fromUtf16(Fields::imports) },
-                { QString(), QString::fromUtf16(Fields::prototypes) },
-                { QString(), QString::fromUtf16(Fields::annotations) },
-                { QString(), QString::fromUtf16(Fields::attachedType) },
-                { QString(), QString::fromUtf16(Fields::canonicalFilePath) },
-                { QString(), QString::fromUtf16(Fields::isValid) },
-                { QString(), QString::fromUtf16(Fields::isSingleton) },
-                { QString(), QString::fromUtf16(Fields::isCreatable) },
-                { QString(), QString::fromUtf16(Fields::isComposite) },
-                { QString(), QString::fromUtf16(Fields::attachedTypeName) },
-                { QString(), QString::fromUtf16(Fields::pragmas) },
-                { QString(), QString::fromUtf16(Fields::defaultPropertyName) },
-                { QString(), QString::fromUtf16(Fields::name) },
-                { QString(), QString::fromUtf16(Fields::nameIdentifiers) },
-                { QString(), QString::fromUtf16(Fields::prototypes) },
-                { QString(), QString::fromUtf16(Fields::nextScope) },
-                { QString(), QString::fromUtf16(Fields::parameters) },
-                { QString(), QString::fromUtf16(Fields::methodType) },
-                { QString(), QString::fromUtf16(Fields::type) },
-                { QString(), QString::fromUtf16(Fields::isConstructor) },
-                { QString(), QString::fromUtf16(Fields::returnType) },
-                { QString(), QString::fromUtf16(Fields::body) },
-                { QString(), QString::fromUtf16(Fields::access) },
-                { QString(), QString::fromUtf16(Fields::typeName) },
-                { QString(), QString::fromUtf16(Fields::isReadonly) },
-                { QString(), QString::fromUtf16(Fields::isList) },
-                { QString(), QString::fromUtf16(Fields::bindingIdentifiers) },
-                { QString(), QString::fromUtf16(Fields::bindingType) },
-                { QString(), QString::fromUtf16(Fields::isSignalHandler) },
+                { QString(), Fields::errors.toString() },
+                { QString(), Fields::imports.toString() },
+                { QString(), Fields::prototypes.toString() },
+                { QString(), Fields::annotations.toString() },
+                { QString(), Fields::attachedType.toString() },
+                { QString(), Fields::canonicalFilePath.toString() },
+                { QString(), Fields::isValid.toString() },
+                { QString(), Fields::isSingleton.toString() },
+                { QString(), Fields::isCreatable.toString() },
+                { QString(), Fields::isComposite.toString() },
+                { QString(), Fields::attachedTypeName.toString() },
+                { QString(), Fields::pragmas.toString() },
+                { QString(), Fields::defaultPropertyName.toString() },
+                { QString(), Fields::name.toString() },
+                { QString(), Fields::nameIdentifiers.toString() },
+                { QString(), Fields::prototypes.toString() },
+                { QString(), Fields::nextScope.toString() },
+                { QString(), Fields::parameters.toString() },
+                { QString(), Fields::methodType.toString() },
+                { QString(), Fields::type.toString() },
+                { QString(), Fields::isConstructor.toString() },
+                { QString(), Fields::returnType.toString() },
+                { QString(), Fields::body.toString() },
+                { QString(), Fields::access.toString() },
+                { QString(), Fields::typeName.toString() },
+                { QString(), Fields::isReadonly.toString() },
+                { QString(), Fields::isList.toString() },
+                { QString(), Fields::bindingIdentifiers.toString() },
+                { QString(), Fields::bindingType.toString() },
+                { QString(), Fields::isSignalHandler.toString() },
                 // prop def?
-                { QString(), QString::fromUtf16(Fields::isPointer) },
-                { QString(), QString::fromUtf16(Fields::isFinal) },
-                { QString(), QString::fromUtf16(Fields::isAlias) },
-                { QString(), QString::fromUtf16(Fields::isDefaultMember) },
-                { QString(), QString::fromUtf16(Fields::isRequired) },
-                { QString(), QString::fromUtf16(Fields::read) },
-                { QString(), QString::fromUtf16(Fields::write) },
-                { QString(), QString::fromUtf16(Fields::bindable) },
-                { QString(), QString::fromUtf16(Fields::notify) },
-                { QString(), QString::fromUtf16(Fields::type) },
+                { QString(), Fields::isPointer.toString() },
+                { QString(), Fields::isFinal.toString() },
+                { QString(), Fields::isAlias.toString() },
+                { QString(), Fields::isDefaultMember.toString() },
+                { QString(), Fields::isRequired.toString() },
+                { QString(), Fields::read.toString() },
+                { QString(), Fields::write.toString() },
+                { QString(), Fields::bindable.toString() },
+                { QString(), Fields::notify.toString() },
+                { QString(), Fields::type.toString() },
                 // scriptExpr
-                { QString(), QString::fromUtf16(Fields::scriptElement) },
-                { QString(), QString::fromUtf16(Fields::localOffset) },
-                { QString(), QString::fromUtf16(Fields::astRelocatableDump) },
-                { QString(), QString::fromUtf16(Fields::expressionType) },
+                { QString(), Fields::scriptElement.toString() },
+                { QString(), Fields::localOffset.toString() },
+                { QString(), Fields::astRelocatableDump.toString() },
+                { QString(), Fields::expressionType.toString() },
                 // components
-                { QString(), QString::fromUtf16(Fields::subComponents) },
+                { QString(), Fields::subComponents.toString() },
                 // BEWARE
                 // Ids and IdStr are filtered out during the visit, because
                 // documentSymbol-s for them will be explicitly handled as part of the
                 // creation of symbol for QmlObject
-                { QString(), QString::fromUtf16(Fields::ids) },
-                { QString(), QString::fromUtf16(Fields::idStr) },
+                { QString(), Fields::ids.toString() },
+                { QString(), Fields::idStr.toString() },
 
                 // id
-                { QString(), QString::fromUtf16(Fields::referredObject) },
+                { QString(), Fields::referredObject.toString() },
                 // enum item
-                { QLatin1String("EnumItem"), QString::fromUtf16(Fields::value) },
+                { QLatin1String("EnumItem"), Fields::value.toString() },
         }
     };
     return ff;
