@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qaccessiblequickitem_p.h"
 
@@ -459,9 +460,34 @@ QAccessible::State QAccessibleQuickItem::state() const
         state.focusable = false;
         state.disabled = true;
     }
+    if (item()->property("readOnly").toBool() || role() == QAccessible::ProgressBar)
+        state.readOnly = true;
     return state;
 }
 
+QList<std::pair<QAccessibleInterface *, QAccessible::Relation>>
+QAccessibleQuickItem::relations(QAccessible::Relation match) const
+{
+    QList<std::pair<QAccessibleInterface *, QAccessible::Relation>> rels =
+            QAccessibleObject::relations(match);
+    if (QQuickAccessibleAttached *attached = QQuickAccessibleAttached::attachedProperties(item())) {
+        if (match & QAccessible::Labelled) {
+            if (auto *labelFor = attached->labelFor()) {
+                rels.append({QAccessible::queryAccessibleInterface(labelFor),
+                             QAccessible::Labelled});
+            }
+        }
+
+        if (match & QAccessible::Label) {
+            if (auto *labelledBy = attached->labelledBy()) {
+                rels.append({QAccessible::queryAccessibleInterface(labelledBy),
+                             QAccessible::Label});
+            }
+        }
+    }
+    return rels;
+}
+
 QAccessible::Role QAccessibleQuickItem::role() const
 {
     // Workaround for setAccessibleRole() not working for
@@ -494,6 +520,7 @@ QStringList QAccessibleQuickItem::actionNames() const
     switch (role()) {
     case QAccessible::Link:
     case QAccessible::PushButton:
+    case QAccessible::MenuItem:
         actions << QAccessibleActionInterface::pressAction();
         break;
     case QAccessible::RadioButton:
@@ -681,20 +708,28 @@ void QAccessibleQuickItem::setText(QAccessible::Text textType, const QString &te
 
 void *QAccessibleQuickItem::interface_cast(QAccessible::InterfaceType t)
 {
-    QAccessible::Role r = role();
-    if (t == QAccessible::ActionInterface)
+    const QAccessible::Role r = role();
+    switch (t) {
+    case QAccessible::ActionInterface:
         return static_cast<QAccessibleActionInterface*>(this);
-    if (t == QAccessible::ValueInterface &&
-           (r == QAccessible::Slider ||
-            r == QAccessible::SpinBox ||
-            r == QAccessible::Dial ||
-            r == QAccessible::ScrollBar))
-       return static_cast<QAccessibleValueInterface*>(this);
-
-    if (t == QAccessible::TextInterface) {
-        if (r == QAccessible::EditableText ||
-            r == QAccessible::StaticText)
-        return static_cast<QAccessibleTextInterface*>(this);
+    case QAccessible::ValueInterface:
+        if (r == QAccessible::Slider
+         || r == QAccessible::SpinBox
+         || r == QAccessible::Dial
+         || r == QAccessible::ScrollBar
+         || r == QAccessible::ProgressBar) {
+            return static_cast<QAccessibleValueInterface*>(this);
+        }
+        break;
+    case QAccessible::TextInterface:
+        if (r == QAccessible::EditableText
+         || r == QAccessible::StaticText
+         || r == QAccessible::Heading) {
+            return static_cast<QAccessibleTextInterface*>(this);
+        }
+        break;
+    default:
+        break;
     }
 
     return QAccessibleObject::interface_cast(t);
@@ -712,12 +747,34 @@ void QAccessibleQuickItem::setCurrentValue(const QVariant &value)
 
 QVariant QAccessibleQuickItem::maximumValue() const
 {
-    return item()->property("maximumValue");
+    const auto minimumValue = item()->property("minimumValue");
+    const auto maximumValue = item()->property("maximumValue");
+    const auto from = item()->property("from");
+    const auto to   = item()->property("to");
+
+    if (minimumValue.isValid() && maximumValue.isValid())
+        return maximumValue;
+
+    if (from.isValid() && to.isValid())
+        return to;
+
+    return QVariant();
 }
 
 QVariant QAccessibleQuickItem::minimumValue() const
 {
-    return item()->property("minimumValue");
+    const auto minimumValue = item()->property("minimumValue");
+    const auto maximumValue = item()->property("maximumValue");
+    const auto from = item()->property("from");
+    const auto to   = item()->property("to");
+
+    if (minimumValue.isValid() && maximumValue.isValid())
+        return minimumValue;
+
+    if (from.isValid() && to.isValid())
+        return from;
+
+    return QVariant();
 }
 
 QVariant QAccessibleQuickItem::minimumStepSize() const
@@ -812,7 +869,7 @@ QString QAccessibleQuickItem::textBeforeOffset(int offset, QAccessible::TextBoun
     if (m_doc) {
         QTextCursor cursor = QTextCursor(m_doc);
         cursor.setPosition(offset);
-        QPair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
+        std::pair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
         cursor.setPosition(boundaries.first - 1);
         boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
 
@@ -834,7 +891,7 @@ QString QAccessibleQuickItem::textAfterOffset(int offset, QAccessible::TextBound
     if (m_doc) {
         QTextCursor cursor = QTextCursor(m_doc);
         cursor.setPosition(offset);
-        QPair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
+        std::pair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
         cursor.setPosition(boundaries.second);
         boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
 
@@ -856,7 +913,7 @@ QString QAccessibleQuickItem::textAtOffset(int offset, QAccessible::TextBoundary
     if (m_doc) {
         QTextCursor cursor = QTextCursor(m_doc);
         cursor.setPosition(offset);
-        QPair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
+        std::pair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
 
         *startOffset = boundaries.first;
         *endOffset = boundaries.second;
@@ -884,9 +941,9 @@ int QAccessibleQuickItem::selectionCount() const
     return 0;
 }
 
-void QAccessibleQuickItem::addSelection(int /* startOffset */, int /* endOffset */)
+void QAccessibleQuickItem::addSelection(int startOffset, int endOffset)
 {
-
+    setSelection(0, startOffset, endOffset);
 }
 void QAccessibleQuickItem::removeSelection(int /* selectionIndex */)
 {
