@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <private/qquickshadereffect_p_p.h>
 #include <private/qsgcontextplugin_p.h>
@@ -567,6 +568,11 @@ QQuickShaderEffect::~QQuickShaderEffect()
     via the Qt resource system. The URL may be absolute, or relative to the URL
     of the component.
 
+    \warning Shaders, including \c{.qsb} files, are assumed to be trusted
+    content. Application developers are advised to carefully consider the
+    potential implications before allowing the loading of user-provided content
+    that is not part of the application.
+
     \sa vertexShader
 */
 
@@ -592,6 +598,11 @@ void QQuickShaderEffect::setFragmentShader(const QUrl &fileUrl)
     via the Qt resource system. The URL may be absolute, or relative to the URL
     of the component.
 
+    \warning Shaders, including \c{.qsb} files, are assumed to be trusted
+    content. Application developers are advised to carefully consider the
+    potential implications before allowing the loading of user-provided content
+    that is not part of the application.
+
     \sa fragmentShader
 */
 
@@ -1171,7 +1182,9 @@ QSGGuiThreadShaderEffectManager *QQuickShaderEffectPrivate::shaderEffectManager(
             return m_mgr;
         QQuickWindow *w = q->window();
         if (w) { // note: just the window, don't care about isSceneGraphInitialized() here
-            m_mgr = QQuickWindowPrivate::get(w)->context->sceneGraphContext()->createGuiThreadShaderEffectManager();
+            QSGRenderContext *renderContext = QQuickWindowPrivate::get(w)->context;
+            if (QSGContext *sgContext = renderContext->sceneGraphContext())
+                m_mgr = sgContext->createGuiThreadShaderEffectManager();
             if (m_mgr) {
                 QObject::connect(m_mgr, &QSGGuiThreadShaderEffectManager::logAndStatusChanged, q, &QQuickShaderEffect::logChanged);
                 QObject::connect(m_mgr, &QSGGuiThreadShaderEffectManager::logAndStatusChanged, q, &QQuickShaderEffect::statusChanged);
