@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickfolderbreadcrumbbar_p.h"
 #include "qquickfolderbreadcrumbbar_p_p.h"
@@ -98,7 +99,7 @@ void QQuickFolderBreadcrumbBarPrivate::repopulate()
         return;
     }
 
-    QBoolBlocker repopulateGuard(repopulating);
+    QScopedValueRollback repopulateGuard(repopulating, true);
 
     auto failureCleanup = [this, q](){
         folderPaths.clear();
