@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquicklayout_p.h"
 #include <QEvent>
@@ -307,6 +308,8 @@ void QQuickLayoutAttached::setMaximumImplicitSize(const QSizeF &sz)
     set to the preferred width.
     The default depends on implicit (built-in) size policy of item.
 
+    \note By default, this property is \c true for layouts.
+
     \sa fillHeight
 */
 void QQuickLayoutAttached::setFillWidth(bool fill)
@@ -328,6 +331,8 @@ void QQuickLayoutAttached::setFillWidth(bool fill)
     set to the preferred height.
     The default depends on implicit (built-in) size policy of the item.
 
+    \note By default, this property is \c true for layouts.
+
     \sa fillWidth
 */
 void QQuickLayoutAttached::setFillHeight(bool fill)
@@ -948,12 +953,30 @@ void QQuickLayout::itemChange(ItemChange change, const ItemChangeData &value)
 void QQuickLayout::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     Q_D(QQuickLayout);
+    qCDebug(lcQuickLayouts) << "QQuickLayout::geometryChange"
+        << oldGeometry << "-->" << newGeometry;
+
     QQuickItem::geometryChange(newGeometry, oldGeometry);
+
     if ((invalidated() && !qobject_cast<QQuickLayout *>(parentItem())) ||
         d->m_disableRearrange || !isReady())
         return;
 
-    qCDebug(lcQuickLayouts) << "QQuickLayout::geometryChange" << newGeometry << oldGeometry;
+    // The geometryChange call above might recursively update the
+    // geometry of this layout, via item change listeners, in which
+    // case the recursive call has already rearranged the layout for
+    // the new size. We don't want to rearrange here based on the old
+    // 'new geometry', as that would revert the most up to date layout.
+    const qreal w = d->width.valueBypassingBindings();
+    const qreal h = d->height.valueBypassingBindings();
+    const QSizeF currentSize(w, h);
+    if (currentSize != newGeometry.size()) {
+        qCDebug(lcQuickLayouts) << "QQuickItem::geometryChange resulted"
+            << "in size change from" << newGeometry.size() << "to"
+            << currentSize << "; layout should already be up to date.";
+        return;
+    }
+
     rearrange(newGeometry.size());
 }
 
