@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QSTYLE_H
 #define QSTYLE_H
@@ -266,6 +267,7 @@ public:
         SE_ScrollBarLayoutItem,
         SE_SpinBoxLayoutItem,
         SE_ToolButtonLayoutItem,
+        SE_SearchFieldLayoutItem,
 
         SE_FrameLayoutItem,
         SE_GroupBoxLayoutItem,
@@ -302,6 +304,7 @@ public:
         CC_Dial,
         CC_GroupBox,
         CC_MdiControls,
+        CC_SearchField,
 
         // do not add any values below/greater than this
         CC_CustomBase = 0xf0000000
@@ -330,6 +333,12 @@ public:
         SC_ComboBoxArrow =         0x00000004,
         SC_ComboBoxListBoxPopup =  0x00000008,
 
+        SC_SearchFieldSearch =     0x00000001,
+        SC_SearchFieldClear =      0x00000002,
+        SC_SearchFieldFrame =      0x00000004,
+        SC_SearchFieldEditField =  0x00000008,
+        SC_SearchFieldPopup =      0x00000010,
+
         SC_SliderGroove =          0x00000001,
         SC_SliderHandle =          0x00000002,
         SC_SliderTickmarks =       0x00000004,
@@ -376,6 +385,7 @@ public:
         PM_DefaultFrameWidth,
         PM_SpinBoxFrameWidth,
         PM_ComboBoxFrameWidth,
+        PM_SearchFieldFrameWidth,
 
         PM_MaximumDragDistance,
 
@@ -496,6 +506,7 @@ public:
         PM_ComboBoxFocusFrameRadius,
         PM_DialFocusFrameRadius,
         PM_RadioButtonFocusFrameRadius,
+        PM_SearchFieldFocusFrameRadius,
         PM_SliderFocusFrameRadius,
         PM_SpinBoxFocusFrameRadius,
         PM_TextAreaFocusFrameRadius,
@@ -525,6 +536,7 @@ public:
         CT_LineEdit,
         CT_SpinBox,
         CT_SizeGrip,
+        CT_SearchField,
         CT_TabWidget,
         CT_DialogButtons,
         CT_HeaderSection,
@@ -781,6 +793,7 @@ public:
     virtual QPixmap standardPixmap(StandardPixmap standardPixmap, const QStyleOption *opt = nullptr) const = 0;
     virtual QIcon standardIcon(StandardPixmap standardIcon, const QStyleOption *option = nullptr) const = 0;
     virtual QPixmap generatedIconPixmap(QIcon::Mode iconMode, const QPixmap &pixmap, const QStyleOption *opt) const = 0;
+    virtual void polish() { }
 
     static QRect visualRect(Qt::LayoutDirection direction, const QRect &boundingRect, const QRect &logicalRect);
     static QPoint visualPos(Qt::LayoutDirection direction, const QRect &boundingRect, const QPoint &logicalPos);
@@ -792,6 +805,8 @@ public:
     // TODO: Remove the concept of proxy (but keep it for now until everything builds)
     const QStyle *proxy() const { return this; }
 
+    virtual void handleThemeChange() {}
+
 private:
     Q_DISABLE_COPY(QStyle)
 };
