@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QSTYLEOPTION_H
 #define QSTYLEOPTION_H
@@ -9,11 +10,15 @@
 #include <QtCore/qlocale.h>
 #include <QtCore/qvariant.h>
 #include <QtCore/qdebug.h>
-#include <QtCore/qabstractitemmodel.h>
 
 #include <QtGui/qicon.h>
 #include <QtGui/qfontmetrics.h>
 
+#include <QtQuick/private/qtquick-config_p.h>
+#if QT_CONFIG(quick_itemview)
+#include <QtCore/qabstractitemmodel.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 class QQuickItem;
@@ -30,7 +35,7 @@ public:
         SO_TabBarBase, SO_RubberBand, SO_ToolBar, SO_GraphicsItem,
 
         SO_Complex = 0xf0000, SO_Slider, SO_SpinBox, SO_ToolButton, SO_ComboBox,
-        SO_TitleBar, SO_GroupBox, SO_SizeGrip,
+        SO_TitleBar, SO_GroupBox, SO_SizeGrip, SO_SearchField,
 
         SO_CustomBase = 0xf00,
         SO_ComplexCustomBase = 0xf000000
@@ -375,6 +380,7 @@ protected:
 
 Q_DECL_DEPRECATED typedef QStyleOptionDockWidget QStyleOptionDockWidgetV2;
 
+#if QT_CONFIG(quick_itemview)
 class QStyleOptionViewItem : public QStyleOption
 {
 public:
@@ -428,6 +434,7 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QStyleOptionViewItem::ViewItemFeatures)
 Q_DECL_DEPRECATED typedef QStyleOptionViewItem QStyleOptionViewItemV2;
 Q_DECL_DEPRECATED typedef QStyleOptionViewItem QStyleOptionViewItemV3;
 Q_DECL_DEPRECATED typedef QStyleOptionViewItem QStyleOptionViewItemV4;
+#endif // QT_CONFIG(quick_itemview)
 
 class QStyleOptionToolBox : public QStyleOption
 {
@@ -594,6 +601,27 @@ protected:
     QStyleOptionComboBox(int version);
 };
 
+
+class QStyleOptionSearchField : public QStyleOptionComplex
+{
+public:
+    enum StyleOptionType { Type = SO_SearchField };
+    enum StyleOptionVersion { Version = 1 };
+
+    QRect popupRect;
+    bool frame;
+    QString text;
+    QIcon currentIcon;
+    QSize iconSize;
+
+    QStyleOptionSearchField();
+    QStyleOptionSearchField(const QStyleOptionSearchField &other) : QStyleOptionComplex(Version, Type) { *this = other;}
+    QStyleOptionSearchField &operator=(const QStyleOptionSearchField &) = default;
+
+protected:
+    QStyleOptionSearchField(int version);
+};
+
 class QStyleOptionTitleBar : public QStyleOptionComplex
 {
 public:
