@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickdial_p.h"
 #include "qquickdeferredexecute_p_p.h"
@@ -7,6 +8,7 @@
 #include <QtCore/qmath.h>
 #include <QtQuick/private/qquickflickable_p.h>
 #include <QtQuickTemplates2/private/qquickcontrol_p_p.h>
+#include <QtQuickTemplates2/private/qtquicktemplates2math_p.h>
 
 #include <cmath>
 
@@ -26,6 +28,7 @@ QT_BEGIN_NAMESPACE
     value within a range.
 
     \image qtquickcontrols-dial-no-wrap.gif
+           {Dial without wrap, stopping at ends}
 
     The value of the dial is set with the \l value property. The range is
     set with the \l from and \l to properties. To enable or disable wrapping,
@@ -342,12 +345,6 @@ void QQuickDialPrivate::executeHandle(bool complete)
         quickCompleteDeferred(q, handleName(), handle);
 }
 
-template<typename ...Real>
-static bool areRepresentableAsInteger(Real... numbers) {
-    auto check = [](qreal number) -> bool { return std::nearbyint(number) == number; };
-    return (... && check(numbers));
-}
-
 void QQuickDialPrivate::updateAllValuesAreInteger()
 {
     allValuesAreInteger = areRepresentableAsInteger(to, from, stepSize) && stepSize != 0.0;
@@ -602,7 +599,7 @@ void QQuickDial::setStartAngle(qreal startAngle)
     The \l endAngle has to be bigger than the \l startAngle, smaller than 720
     and smaller or equal than the \l startAngle + 360 degrees.
 
-    \sa endAngle, angle
+    \sa startAngle, angle
 */
 qreal QQuickDial::endAngle() const
 {
@@ -720,11 +717,13 @@ void QQuickDial::setInputMode(QQuickDial::InputMode mode)
     \l from position, and vice versa:
 
     \image qtquickcontrols-dial-wrap.gif
+           {Dial with wrap, rotating continuously}
 
     When this property is \c false, it's not possible to drag the dial across
     the from and to values.
 
     \image qtquickcontrols-dial-no-wrap.gif
+           {Dial without wrap, stopping at ends}
 
     The default value is \c false.
 */
@@ -939,11 +938,13 @@ void QQuickDial::touchEvent(QTouchEvent *event)
             switch (point.state()) {
             case QEventPoint::Updated:
                 if (!keepTouchGrab()) {
-                    bool overXDragThreshold = QQuickWindowPrivate::dragOverThreshold(point.position().x() - d->pressPoint.x(), Qt::XAxis, &point);
+                    bool overXDragThreshold = QQuickDeliveryAgentPrivate::dragOverThreshold(point.position().x() - d->pressPoint.x(),
+                                                                            Qt::XAxis, point);
                     setKeepTouchGrab(overXDragThreshold);
 
                     if (!overXDragThreshold) {
-                        bool overYDragThreshold = QQuickWindowPrivate::dragOverThreshold(point.position().y() - d->pressPoint.y(), Qt::YAxis, &point);
+                        bool overYDragThreshold = QQuickDeliveryAgentPrivate::dragOverThreshold(point.position().y() - d->pressPoint.y(),
+                                                                            Qt::YAxis, point);
                         setKeepTouchGrab(overYDragThreshold);
                     }
                 }
