@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickdrawer_p.h"
 #include "qquickdrawer_p_p.h"
@@ -32,6 +33,7 @@ QT_BEGIN_NAMESPACE
     touch interfaces to provide a central location for navigation.
 
     \image qtquickcontrols-drawer.gif
+           {Drawer sliding in from edge}
 
     Drawer can be positioned at any of the four edges of the content item.
     The drawer above is positioned against the left edge of the window. The
