@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickrangeslider_p.h"
 #include "qquickcontrol_p_p.h"
@@ -7,6 +8,7 @@
 
 #include <QtCore/qscopedpointer.h>
 #include <QtQuick/private/qquickwindow_p.h>
+#include <QtQuickTemplates2/private/qtquicktemplates2math_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -21,6 +23,7 @@ QT_BEGIN_NAMESPACE
     \brief Used to select a range of values by sliding two handles along a track.
 
     \image qtquickcontrols-rangeslider.gif
+           {Range slider with two handles defining range}
 
     RangeSlider is used to select a range specified by two values, by sliding
     each handle along a track.
@@ -342,6 +345,8 @@ class QQuickRangeSliderPrivate : public QQuickControlPrivate
     Q_DECLARE_PUBLIC(QQuickRangeSlider)
 
 public:
+    static const QQuickRangeSliderPrivate *get(const QQuickRangeSlider *slider);
+
     QQuickRangeSliderNode *pressedNode(int touchId = -1) const;
 
 #if QT_CONFIG(quicktemplates2_multitouch)
@@ -358,7 +363,8 @@ public:
     void itemImplicitHeightChanged(QQuickItem *item) override;
     void itemDestroyed(QQuickItem *item) override;
 
-    bool live = true;
+    void updateAllValuesAreInteger();
+
     qreal from = defaultFrom;
     qreal to = defaultTo;
     qreal stepSize = 0;
@@ -368,11 +374,19 @@ public:
     QPointF pressPoint;
     Qt::Orientation orientation = Qt::Horizontal;
     QQuickRangeSlider::SnapMode snapMode = QQuickRangeSlider::NoSnap;
+    bool live = true;
+    bool allValuesAreInteger = false;
 };
 
 static qreal valueAt(const QQuickRangeSlider *slider, qreal position)
 {
-    return slider->from() + (slider->to() - slider->from()) * position;
+    qreal value = slider->from() + (slider->to() - slider->from()) * position;
+
+    // See the comment in QQuickDialPrivate::valueAt for why we do this.
+    if (QQuickRangeSliderPrivate::get(slider)->allValuesAreInteger)
+        value = qRound(value);
+
+    return value;
 }
 
 static qreal snapPosition(const QQuickRangeSlider *slider, qreal position)
@@ -409,6 +423,11 @@ static qreal positionAt(const QQuickRangeSlider *slider, QQuickItem *handle, con
     return 0;
 }
 
+const QQuickRangeSliderPrivate *QQuickRangeSliderPrivate::get(const QQuickRangeSlider *slider)
+{
+    return slider->d_func();
+}
+
 QQuickRangeSliderNode *QQuickRangeSliderPrivate::pressedNode(int touchId) const
 {
     if (touchId == -1)
@@ -612,6 +631,11 @@ void QQuickRangeSliderPrivate::itemDestroyed(QQuickItem *item)
         second->setHandle(nullptr);
 }
 
+void QQuickRangeSliderPrivate::updateAllValuesAreInteger()
+{
+    allValuesAreInteger = areRepresentableAsInteger(to, from, stepSize) && stepSize != 0.0;
+}
+
 QQuickRangeSlider::QQuickRangeSlider(QQuickItem *parent)
     : QQuickControl(*(new QQuickRangeSliderPrivate), parent)
 {
@@ -663,6 +687,7 @@ void QQuickRangeSlider::setFrom(qreal from)
 
     d->from = from;
     emit fromChanged();
+    d->updateAllValuesAreInteger();
 
     if (isComponentComplete()) {
         d->first->setValue(d->first->value());
@@ -695,6 +720,7 @@ void QQuickRangeSlider::setTo(qreal to)
 
     d->to = to;
     emit toChanged();
+    d->updateAllValuesAreInteger();
 
     if (isComponentComplete()) {
         d->first->setValue(d->first->value());
@@ -921,6 +947,7 @@ void QQuickRangeSlider::setStepSize(qreal step)
 
     d->stepSize = step;
     emit stepSizeChanged();
+    d->updateAllValuesAreInteger();
 }
 
 /*!
@@ -1220,10 +1247,13 @@ void QQuickRangeSlider::touchEvent(QTouchEvent *event)
                 break;
             case QEventPoint::Updated:
                 if (!keepTouchGrab()) {
-                    if (d->orientation == Qt::Horizontal)
-                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.position().x() - point.pressPosition().x(), Qt::XAxis, &point, qRound(d->touchDragThreshold)));
-                    else
-                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.position().y() - point.pressPosition().y(), Qt::YAxis, &point, qRound(d->touchDragThreshold)));
+                    if (d->orientation == Qt::Horizontal) {
+                        setKeepTouchGrab(QQuickDeliveryAgentPrivate::dragOverThreshold(point.position().x() - point.pressPosition().x(),
+                                            Qt::XAxis, point, qRound(d->touchDragThreshold)));
+                    } else {
+                        setKeepTouchGrab(QQuickDeliveryAgentPrivate::dragOverThreshold(point.position().y() - point.pressPosition().y(),
+                                            Qt::YAxis, point, qRound(d->touchDragThreshold)));
+                    }
                 }
                 if (keepTouchGrab())
                     d->handleMove(point.position(), event->timestamp());
@@ -1294,6 +1324,8 @@ void QQuickRangeSlider::componentComplete()
         firstPrivate->updatePosition();
         secondPrivate->updatePosition();
     }
+
+    d->updateAllValuesAreInteger();
 }
 
 /*!
