@@ -34,9 +34,9 @@
 
 QT_BEGIN_NAMESPACE
 
-using namespace Qt::StringLiterals;
+Q_STATIC_LOGGING_CATEGORY(lcVectorImageAnimations, "qt.quick.vectorimage.animations")
 
-Q_DECLARE_LOGGING_CATEGORY(lcQuickVectorImage)
+using namespace Qt::StringLiterals;
 
 class QSvgStyleResolver
 {
@@ -158,7 +158,8 @@ inline bool isPathContainer(const QSvgStructureNode *node)
         case QSvgNode::Switch:
         case QSvgNode::Doc:
         case QSvgNode::Group:
-        case QSvgNode::Animation:
+        case QSvgNode::AnimateColor:
+        case QSvgNode::AnimateTransform:
         case QSvgNode::Use:
         case QSvgNode::Video:
         case QSvgNode::Image:
@@ -180,12 +181,22 @@ inline bool isPathContainer(const QSvgStructureNode *node)
         case QSvgNode::Path:
         case QSvgNode::Polygon:
         case QSvgNode::Polyline:
+        {
+            if (!child->style().opacity.isDefault())
+                return false;
+
             if (!child->style().transform.isDefault()) {
                 //qCDebug(lcQuickVectorGraphics) << "NOT path container because local transform";
                 return false;
             }
+            const QList<QSvgAbstractAnimation *> animations = child->document()->animator()->animationsForNode(child);
+            if (!animations.isEmpty()) {
+                //qCDebug(lcQuickVectorGraphics) << "NOT path container because local transform animation";
+                return false;
+            }
             foundPath = true;
             break;
+        }
         default:
             qCDebug(lcQuickVectorImage) << "Unhandled type in switch" << child->type();
             break;
@@ -259,9 +270,12 @@ static QString dashArrayString(QList<qreal> dashArray)
 }
 };
 
-QSvgVisitorImpl::QSvgVisitorImpl(const QString svgFileName, QQuickGenerator *generator)
+QSvgVisitorImpl::QSvgVisitorImpl(const QString svgFileName,
+                                 QQuickGenerator *generator,
+                                 bool assumeTrustedSource)
     : m_svgFileName(svgFileName)
     , m_generator(generator)
+    , m_assumeTrustedSource(assumeTrustedSource)
 {
 }
 
@@ -272,7 +286,11 @@ bool QSvgVisitorImpl::traverse()
         return false;
     }
 
-    auto *doc = QSvgTinyDocument::load(m_svgFileName);
+    QtSvg::Options options;
+    if (m_assumeTrustedSource)
+        options.setFlag(QtSvg::AssumeTrustedSource);
+
+    auto *doc = QSvgTinyDocument::load(m_svgFileName, options);
     if (!doc) {
         qCDebug(lcQuickVectorImage) << "Not a valid Svg File : " << m_svgFileName;
         return false;
@@ -288,6 +306,7 @@ void QSvgVisitorImpl::visitNode(const QSvgNode *node)
 
     NodeInfo info;
     fillCommonNodeInfo(node, info);
+    fillAnimationInfo(node, info);
 
     m_generator->generateNode(info);
 
@@ -301,6 +320,7 @@ void QSvgVisitorImpl::visitImageNode(const QSvgImage *node)
 
     ImageNodeInfo info;
     fillCommonNodeInfo(node, info);
+    fillAnimationInfo(node, info);
     info.image = node->image();
     info.rect = node->rect();
     info.externalFileReference = node->filename();
@@ -497,7 +517,8 @@ namespace {
 
     glyph_t QSvgFontEngine::glyphIndex(uint ucs4) const
     {
-        if (ucs4 < USHRT_MAX && m_font->m_glyphs.contains(QChar(ushort(ucs4))))
+        const ushort c(ucs4);
+        if (ucs4 < USHRT_MAX && m_font->findFirstGlyphFor(QStringView(&c, 1)))
             return glyph_t(ucs4);
 
         return 0;
@@ -544,7 +565,13 @@ namespace {
             glyph_t index = glyphs[i];
             if (index > 0) {
                 QPointF position = positions[i].toPointF();
-                QPainterPath glyphPath = m_font->m_glyphs.value(QChar(ushort(index))).m_path;
+                const ushort c(index);
+                const QSvgGlyph *foundGlyph = m_font->findFirstGlyphFor(QStringView(&c, 1));
+
+                if (!foundGlyph)
+                    continue;
+
+                QPainterPath glyphPath = foundGlyph->m_path;
 
                 QTransform xform;
                 xform.translate(position.x(), position.y());
@@ -561,8 +588,9 @@ namespace {
         ret.x = 0; // left bearing
         ret.y = -ascent();
         const qreal scale = fontDef.pixelSize / m_font->m_unitsPerEm;
-        const QSvgGlyph &svgGlyph = m_font->m_glyphs.value(QChar(ushort(glyph)));
-        ret.width = QFixed::fromReal(svgGlyph.m_horizAdvX * scale);
+        const ushort c(glyph);
+        const QSvgGlyph *svgGlyph = m_font->findFirstGlyphFor(QStringView(&c, 1));
+        ret.width = QFixed::fromReal(svgGlyph ? svgGlyph->m_horizAdvX * scale : 0.);
         ret.height = ascent() + descent();
         return ret;
     }
@@ -577,9 +605,9 @@ namespace {
     {
         const qreal scale = fontDef.pixelSize / m_font->m_unitsPerEm;
         for (int i = 0; i < glyphLayout->numGlyphs; i++) {
-            glyph_t glyph = glyphLayout->glyphs[i];
-            const QSvgGlyph &svgGlyph = m_font->m_glyphs.value(QChar(ushort(glyph)));
-            glyphLayout->advances[i] = QFixed::fromReal(svgGlyph.m_horizAdvX * scale);
+            const ushort c(glyphLayout->glyphs[i]);
+            const QSvgGlyph *svgGl = m_font->findFirstGlyphFor(QStringView(&c, 1));
+            glyphLayout->advances[i] = QFixed::fromReal(svgGl ? svgGl->m_horizAdvX * scale : 0.);
         }
     }
 
@@ -619,6 +647,16 @@ namespace {
     }
 }
 
+static QVariant calculateInterpolatedValue(const QSvgAbstractAnimatedProperty *property, int index, int)
+{
+    if (index == 0)
+        const_cast<QSvgAbstractAnimatedProperty *>(property)->interpolate(1, 0.0);
+    else
+        const_cast<QSvgAbstractAnimatedProperty *>(property)->interpolate(index, 1.0);
+
+    return property->interpolatedValue();
+}
+
 void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
 {
     handleBaseNodeSetup(node);
@@ -728,7 +766,7 @@ void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
 
         needsRichText = needsRichText || !styleTagContent.isEmpty();
         if (!styleTagContent.isEmpty())
-            text += QStringLiteral("<span style=\"%1\">").arg(styleTagContent);
+            text += QStringLiteral("<span style=\"%1\">").arg(styleTagContent.toHtmlEscaped());
 
         if (font.resolveMask() & QFont::WeightResolved && font.bold())
             text += QStringLiteral("<b>");
@@ -811,16 +849,17 @@ void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
                 auto addPathForFormat = [&](QPainterPath p, QTextCharFormat fmt) {
                     PathNodeInfo info;
                     fillCommonNodeInfo(node, info);
+                    fillPathAnimationInfo(node, info);
                     auto fillStyle = node->style().fill;
                     if (fillStyle)
                         info.fillRule = fillStyle->fillRule();
 
                     if (fmt.hasProperty(QTextCharFormat::ForegroundBrush)) {
-                        info.fillColor = fmt.foreground().color();
+                        info.fillColor.setDefaultValue(fmt.foreground().color());
                         if (fmt.foreground().gradient() != nullptr && fmt.foreground().gradient()->type() != QGradient::NoGradient)
                             info.grad = *fmt.foreground().gradient();
                     } else {
-                        info.fillColor = styleResolver->currentFillColor();
+                        info.fillColor.setDefaultValue(styleResolver->currentFillColor());
                     }
 
                     info.painterPath = p;
@@ -831,13 +870,13 @@ void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
                         pen = fmt.textOutline();
                         if (strokeGradient == nullptr) {
                             info.strokeStyle = StrokeStyle::fromPen(pen);
-                            info.strokeStyle.color = pen.color();
+                            info.strokeStyle.color.setDefaultValue(pen.color());
                         }
                     } else {
                         pen = styleResolver->currentStroke();
                         if (strokeGradient == nullptr) {
                             info.strokeStyle = StrokeStyle::fromPen(pen);
-                            info.strokeStyle.color = styleResolver->currentStrokeColor();
+                            info.strokeStyle.color.setDefaultValue(styleResolver->currentStrokeColor());
                         }
                     }
 
@@ -851,6 +890,7 @@ void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
                     if (strokeGradient != nullptr) {
                         PathNodeInfo strokeInfo;
                         fillCommonNodeInfo(node, strokeInfo);
+                        fillPathAnimationInfo(node, strokeInfo);
 
                         strokeInfo.grad = *strokeGradient;
 
@@ -893,15 +933,6 @@ void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
                 for (int i = 0; i < formats.size(); ++i) {
                     QTextLayout::FormatRange range = formats.at(i);
 
-                    // If we hit a "multi" anchor, it means we have additional formats to apply
-                    // for both this and the subsequent range, so we merge them.
-                    if (!range.format.anchorNames().isEmpty()
-                        && range.format.anchorNames().first().startsWith(QStringLiteral("multi"))
-                        && i < formats.size() - 1) {
-                        QTextLayout::FormatRange nextRange = formats.at(++i);
-                        range.length += nextRange.length;
-                        range.format.merge(nextRange.format);
-                    }
                     QList<QGlyphRun> glyphRuns = lout->glyphRuns(range.start, range.length);
                     QList<QPainterPath> paths = glyphsToPath(glyphRuns, lout->minimumWidth());
                     for (const QPainterPath &path : paths)
@@ -916,6 +947,31 @@ void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
     {
         TextNodeInfo info;
         fillCommonNodeInfo(node, info);
+        fillAnimationInfo(node, info);
+
+        {
+            QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("fill"));
+            if (!animations.isEmpty())
+                applyAnimationsToProperty(animations, &info.fillColor, calculateInterpolatedValue);
+        }
+
+        {
+            QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("fill-opacity"));
+            if (!animations.isEmpty())
+                applyAnimationsToProperty(animations, &info.fillOpacity, calculateInterpolatedValue);
+        }
+
+        {
+            QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("stroke"));
+            if (!animations.isEmpty())
+                applyAnimationsToProperty(animations, &info.strokeColor, calculateInterpolatedValue);
+        }
+
+        {
+            QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("stroke-opacity"));
+            if (!animations.isEmpty())
+                applyAnimationsToProperty(animations, &info.strokeOpacity, calculateInterpolatedValue);
+        }
 
         info.position = node->position();
         info.size = node->size();
@@ -923,9 +979,9 @@ void QSvgVisitorImpl::visitTextNode(const QSvgText *node)
         info.text = text;
         info.isTextArea = isTextArea;
         info.needsRichText = needsRichText;
-        info.fillColor = styleResolver->currentFillColor();
+        info.fillColor.setDefaultValue(styleResolver->currentFillColor());
         info.alignment = styleResolver->states().textAnchor;
-        info.strokeColor = styleResolver->currentStrokeColor();
+        info.strokeColor.setDefaultValue(styleResolver->currentStrokeColor());
 
         m_generator->generateTextNode(info);
     }
@@ -948,9 +1004,19 @@ void QSvgVisitorImpl::visitUseNode(const QSvgUse *node)
     handleBaseNodeSetup(node);
     UseNodeInfo info;
     fillCommonNodeInfo(node, info);
+    fillAnimationInfo(node, info);
 
     info.stage = StructureNodeStage::Start;
-    info.startPos = node->start();
+
+    QPointF startPos = node->start();
+    if (!startPos.isNull()) {
+        QTransform xform;
+        if (!info.isDefaultTransform)
+            xform = info.transform.defaultValue().value<QTransform>();
+        xform.translate(startPos.x(), startPos.y());
+        info.transform.setDefaultValue(QVariant::fromValue(xform));
+        info.isDefaultTransform = false;
+    }
 
     m_generator->generateUseNode(info);
 
@@ -992,6 +1058,7 @@ bool QSvgVisitorImpl::visitStructureNodeStart(const QSvgStructureNode *node)
     StructureNodeInfo info;
 
     fillCommonNodeInfo(node, info);
+    fillAnimationInfo(node, info);
     info.forceSeparatePaths = forceSeparatePaths;
     info.isPathContainer = isPathContainer(node);
     info.stage = StructureNodeStage::Start;
@@ -1007,6 +1074,7 @@ void QSvgVisitorImpl::visitStructureNodeEnd(const QSvgStructureNode *node)
 
     StructureNodeInfo info;
     fillCommonNodeInfo(node, info);
+    info.isPathContainer = isPathContainer(node);
     info.stage = StructureNodeStage::End;
 
     m_generator->generateStructureNode(info);
@@ -1018,6 +1086,7 @@ bool QSvgVisitorImpl::visitDocumentNodeStart(const QSvgTinyDocument *node)
 
     StructureNodeInfo info;
     fillCommonNodeInfo(node, info);
+    fillAnimationInfo(node, info);
 
     const QSvgTinyDocument *doc = static_cast<const QSvgTinyDocument *>(node);
     info.size = doc->size();
@@ -1043,18 +1112,253 @@ void QSvgVisitorImpl::visitDocumentNodeEnd(const QSvgTinyDocument *node)
     m_generator->generateRootNode(info);
 }
 
+static QString scrub(const QString &raw)
+{
+    QString res(raw.left(80));
+
+    if (!res.isEmpty()) {
+        constexpr QLatin1StringView legalSymbols("_-.:"); // Only valid SVG id characters
+        qsizetype i = 0;
+        do {
+            if (res.at(i).isLetterOrNumber() || legalSymbols.contains(res.at(i)))
+                i++;
+            else
+                res.remove(i, 1);
+        } while (i < res.size());
+    }
+
+    return res;
+}
+
 void QSvgVisitorImpl::fillCommonNodeInfo(const QSvgNode *node, NodeInfo &info)
 {
-    info.nodeId = node->nodeId();
+    info.nodeId = scrub(node->nodeId());
     info.typeName = node->typeName();
     info.isDefaultTransform = node->style().transform.isDefault();
-    info.transform = !info.isDefaultTransform ? node->style().transform->qtransform() : QTransform();
+    info.transform.setDefaultValue(QVariant::fromValue(!info.isDefaultTransform
+                                                           ? node->style().transform->qtransform()
+                                                           : QTransform()));
     info.isDefaultOpacity = node->style().opacity.isDefault();
-    info.opacity = !info.isDefaultOpacity ? node->style().opacity->opacity() : 1.0;
+    info.opacity.setDefaultValue(!info.isDefaultOpacity ? node->style().opacity->opacity() : 1.0);
     info.isVisible = node->isVisible();
     info.isDisplayed = node->displayMode() != QSvgNode::DisplayMode::NoneMode;
 }
 
+QList<QSvgVisitorImpl::AnimationPair> QSvgVisitorImpl::collectAnimations(const QSvgNode *node,
+                                                                         const QString &propertyName)
+{
+    QList<AnimationPair> ret;
+    const QList<QSvgAbstractAnimation *> animations = node->document()->animator()->animationsForNode(node);
+    for (const QSvgAbstractAnimation *animation : animations) {
+        const QList<QSvgAbstractAnimatedProperty *> properties = animation->properties();
+        for (const QSvgAbstractAnimatedProperty *property : properties) {
+            if (property->propertyName() == propertyName)
+                ret.append(std::make_pair(animation, property));
+        }
+    }
+
+    return ret;
+}
+
+void QSvgVisitorImpl::applyAnimationsToProperty(const QList<AnimationPair> &animations,
+                                                QQuickAnimatedProperty *outProperty,
+                                                std::function<QVariant(const QSvgAbstractAnimatedProperty *, int index, int animationIndex)> calculateValue)
+{
+    qCDebug(lcVectorImageAnimations) << "Applying animations to property with default value"
+                                     << outProperty->defaultValue();
+    for (auto it = animations.constBegin(); it != animations.constEnd(); ++it) {
+        qCDebug(lcVectorImageAnimations) << "    -> Add animation";
+        const QSvgAbstractAnimation *animation = it->first;
+        const QSvgAbstractAnimatedProperty *property = it->second;
+
+        const int start = animation->start();
+        const int repeatCount = animation->iterationCount();
+        const int duration = animation->duration();
+
+        bool freeze = false;
+        bool replace = true;
+        if (animation->animationType() == QSvgAbstractAnimation::SMIL) {
+            const QSvgAnimateNode *animateNode = static_cast<const QSvgAnimateNode *>(animation);
+            freeze = animateNode->fill() == QSvgAnimateNode::Freeze;
+            replace = animateNode->additiveType() == QSvgAnimateNode::Replace;
+        }
+
+        qCDebug(lcVectorImageAnimations) << "        -> Start:" << start
+                                         << ", repeatCount:" << repeatCount
+                                         << ", freeze:" << freeze
+                                         << ", replace:" << replace;
+
+        QList<qreal> propertyKeyFrames = property->keyFrames();
+        QList<QQuickAnimatedProperty::PropertyAnimation> outAnimations;
+
+        // For transform animations, we register the type of the transform in the animation
+        // (this assumes that each animation is only for a single part of the transform)
+        if (property->type() == QSvgAbstractAnimatedProperty::Transform) {
+            const auto *transformProperty = static_cast<const QSvgAnimatedPropertyTransform *>(property);
+            const auto &components = transformProperty->components();
+            Q_ASSERT(q20::cmp_greater_equal(components.size(),transformProperty->transformCount()));
+            for (uint i = 0; i < transformProperty->transformCount(); ++i) {
+                QQuickAnimatedProperty::PropertyAnimation outAnimation;
+                outAnimation.repeatCount = repeatCount;
+                outAnimation.startOffset = start;
+                if (freeze)
+                    outAnimation.flags |= QQuickAnimatedProperty::PropertyAnimation::FreezeAtEnd;
+                if (replace)
+                    outAnimation.flags |= QQuickAnimatedProperty::PropertyAnimation::ReplacePreviousAnimations;
+                switch (components.at(i).type) {
+                case QSvgAnimatedPropertyTransform::TransformComponent::Translate:
+                    outAnimation.subtype = QTransform::TxTranslate;
+                    break;
+                case QSvgAnimatedPropertyTransform::TransformComponent::Scale:
+                    outAnimation.subtype = QTransform::TxScale;
+                    break;
+                case QSvgAnimatedPropertyTransform::TransformComponent::Rotate:
+                    outAnimation.subtype = QTransform::TxRotate;
+                    break;
+                case QSvgAnimatedPropertyTransform::TransformComponent::Skew:
+                    outAnimation.subtype = QTransform::TxShear;
+                    break;
+                default:
+                    qCWarning(lcQuickVectorImage()) << "Unhandled transform type:" << components.at(i).type;
+                    break;
+                }
+
+                qDebug(lcVectorImageAnimations) << "        -> Property type:"
+                                                << property->type()
+                                                << " name:"
+                                                << property->propertyName()
+                                                << " animation subtype:"
+                                                << outAnimation.subtype;
+
+                outAnimations.append(outAnimation);
+            }
+        } else {
+            QQuickAnimatedProperty::PropertyAnimation outAnimation;
+            outAnimation.repeatCount = repeatCount;
+            outAnimation.startOffset = start;
+            if (freeze)
+                outAnimation.flags |= QQuickAnimatedProperty::PropertyAnimation::FreezeAtEnd;
+
+            qDebug(lcVectorImageAnimations) << "        -> Property type:"
+                                            << property->type()
+                                            << " name:"
+                                            << property->propertyName();
+
+            outAnimations.append(outAnimation);
+        }
+
+        outProperty->beginAnimationGroup();
+        for (int i = 0; i < outAnimations.size(); ++i) {
+            QQuickAnimatedProperty::PropertyAnimation outAnimation = outAnimations.at(i);
+
+            for (int j = 0; j < propertyKeyFrames.size(); ++j) {
+                const int time = qRound(propertyKeyFrames.at(j) * duration);
+
+                const QVariant value = calculateValue(property, j, i);
+                outAnimation.frames[time] = value;
+                qCDebug(lcVectorImageAnimations) << "        -> Frame " << time << " is " << value;
+            }
+
+            outProperty->addAnimation(outAnimation);
+        }
+    }
+}
+
+void QSvgVisitorImpl::fillColorAnimationInfo(const QSvgNode *node, PathNodeInfo &info)
+{
+    // Collect all animations affecting fill
+    {
+        QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("fill"));
+        if (!animations.isEmpty())
+            applyAnimationsToProperty(animations, &info.fillColor, calculateInterpolatedValue);
+    }
+
+    {
+        QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("fill-opacity"));
+        if (!animations.isEmpty())
+            applyAnimationsToProperty(animations, &info.fillOpacity, calculateInterpolatedValue);
+    }
+
+    {
+        QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("stroke"));
+        if (!animations.isEmpty())
+            applyAnimationsToProperty(animations, &info.strokeStyle.color, calculateInterpolatedValue);
+    }
+
+    {
+        QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("stroke-opacity"));
+        if (!animations.isEmpty())
+            applyAnimationsToProperty(animations, &info.strokeStyle.opacity, calculateInterpolatedValue);
+    }
+}
+
+void QSvgVisitorImpl::fillTransformAnimationInfo(const QSvgNode *node, NodeInfo &info)
+{
+    qCDebug(lcVectorImageAnimations) << "Applying transform animations to property with default value"
+                                     << info.transform.defaultValue();
+
+    auto calculateValue = [](const QSvgAbstractAnimatedProperty *property, int index, int animationIndex) {
+        if (property->type() != QSvgAbstractAnimatedProperty::Transform)
+            return QVariant{};
+
+        const auto *transformProperty = static_cast<const QSvgAnimatedPropertyTransform *>(property);
+        const auto &components = transformProperty->components();
+
+        const int componentIndex = index * transformProperty->transformCount() + animationIndex;
+
+        QVariantList parameters;
+
+        const QSvgAnimatedPropertyTransform::TransformComponent &component = components.at(componentIndex);
+        switch (component.type) {
+        case QSvgAnimatedPropertyTransform::TransformComponent::Translate:
+            parameters.append(QVariant::fromValue(QPointF(component.values.value(0),
+                                                          component.values.value(1))));
+            break;
+        case QSvgAnimatedPropertyTransform::TransformComponent::Rotate:
+            parameters.append(QVariant::fromValue(QPointF(component.values.value(1),
+                                                          component.values.value(2))));
+            parameters.append(QVariant::fromValue(component.values.value(0)));
+            break;
+        case QSvgAnimatedPropertyTransform::TransformComponent::Scale:
+            parameters.append(QVariant::fromValue(QPointF(component.values.value(0),
+                                                          component.values.value(1))));
+            break;
+        case QSvgAnimatedPropertyTransform::TransformComponent::Skew:
+            parameters.append(QVariant::fromValue(QPointF(component.values.value(0),
+                                                          component.values.value(1))));
+            break;
+        default:
+            qCWarning(lcVectorImageAnimations) << "Unhandled transform type:" << component.type;
+        };
+
+        return QVariant::fromValue(parameters);
+    };
+
+
+    {
+        QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("transform"));
+        if (!animations.isEmpty())
+            applyAnimationsToProperty(animations, &info.transform, calculateValue);
+    }
+}
+
+void QSvgVisitorImpl::fillPathAnimationInfo(const QSvgNode *node, PathNodeInfo &info)
+{
+    fillColorAnimationInfo(node, info);
+    fillAnimationInfo(node, info);
+}
+
+void QSvgVisitorImpl::fillAnimationInfo(const QSvgNode *node, NodeInfo &info)
+{
+    {
+        QList<AnimationPair> animations = collectAnimations(node, QStringLiteral("opacity"));
+        if (!animations.isEmpty())
+            applyAnimationsToProperty(animations, &info.opacity, calculateInterpolatedValue);
+    }
+
+    fillTransformAnimationInfo(node, info);
+}
+
 void QSvgVisitorImpl::handleBaseNodeSetup(const QSvgNode *node)
 {
     qCDebug(lcQuickVectorImage) << "Before SETUP" << node << "fill" << styleResolver->currentFillColor()
@@ -1098,15 +1402,17 @@ void QSvgVisitorImpl::handlePathNode(const QSvgNode *node, const QPainterPath &p
     const QGradient *strokeGradient = styleResolver->currentStrokeGradient();
 
     info.painterPath = path;
-    info.fillColor = styleResolver->currentFillColor();
+    info.fillColor.setDefaultValue(styleResolver->currentFillColor());
     if (strokeGradient == nullptr) {
         info.strokeStyle = StrokeStyle::fromPen(styleResolver->currentStroke());
-        info.strokeStyle.color = styleResolver->currentStrokeColor();
+        info.strokeStyle.color.setDefaultValue(styleResolver->currentStrokeColor());
     }
     if (styleResolver->currentFillGradient() != nullptr)
         info.grad = styleResolver->applyOpacityToGradient(*styleResolver->currentFillGradient(), styleResolver->currentFillOpacity());
     info.fillTransform = styleResolver->currentFillTransform();
 
+    fillPathAnimationInfo(node, info);
+
     m_generator->generatePath(info);
 
     if (strokeGradient != nullptr) {
