@@ -3,9 +3,9 @@
 
 /*---
 esid: sec-array.prototype.reduceright
-es5id: 15.4.4.22-8-b-iii-1-29
+es5id: 15.4.4.22-8-b-iii-1-28
 description: >
-    Array.prototype.reduceRight applied to Function object which
+    Array.prototype.reduceRight applied to String object, which
     implements its own property get method
 ---*/
 
@@ -13,18 +13,12 @@ var testResult = false;
 
 function callbackfn(prevVal, curVal, idx, obj) {
   if (idx === 1) {
-    testResult = (prevVal === 2);
+    testResult = (prevVal === "2");
   }
 }
 
-var obj = function(a, b, c) {
-  return a + b + c;
-};
+var str = new String("012");
 
-obj[0] = 0;
-obj[1] = 1;
-obj[2] = 2;
-
-Array.prototype.reduceRight.call(obj, callbackfn);
+Array.prototype.reduceRight.call(str, callbackfn);
 
 assert(testResult, 'testResult !== true');
