@@ -2,10 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.12-3-19
-description: Object.isFrozen returns false for all built-in objects (Error)
+es5id: 15.2.3.12-3-24
+description: >
+    Object.isFrozen returns false for all built-in objects
+    (SyntaxError)
 ---*/
 
-var b = Object.isFrozen(Error);
+var b = Object.isFrozen(SyntaxError);
 
 assert.sameValue(b, false, 'b');
