@@ -2,18 +2,16 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 14.1-5gs
+es5id: 10.1.1-5gs
 description: >
-    StrictMode - a Use Strict Directive embedded in a directive
-    prologue followed by a strict mode violation
+    Strict Mode - Use Strict Directive Prologue is ''use strict';'
+    which appears at the start of the code
 negative:
   phase: parse
   type: SyntaxError
 flags: [raw]
 ---*/
 
-"a";
 "use strict";
-"c";
 throw new Error("This code should not execute");
-eval = 42;
+var public = 1;
