@@ -3,28 +3,28 @@
 
 /*---
 info: Compound Assignment Operator calls PutValue(lref, v)
-es5id: S11.13.2_A5.8_T5
+es5id: S11.13.2_A5.2_T5
 description: >
     Evaluating LeftHandSideExpression lref returns Reference type; Reference
     base value is an environment record and environment record kind is
     object environment record. PutValue(lref, v) uses the initially
     created Reference even if the environment binding is no longer present.
-    No ReferenceError is thrown when 'x >>>= y' is in strict-mode code and the
+    No ReferenceError is thrown when 'x /= y' is in strict-mode code and the
     original binding is no longer present.
-    Check operator is "x >>>= y".
+    Check operator is "x /= y".
 ---*/
 
 Object.defineProperty(this, "x", {
   configurable: true,
   get: function() {
     delete this.x;
-    return 16;
+    return 6;
   }
 });
 
 (function() {
   "use strict";
-  x >>>= 3;
+  x /= 3;
 })();
 
 if (this.x !== 2) {
