@@ -2,14 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.2.1-37-s
+es5id: 12.2.1-18-s
 description: >
-    'for(var arguments = 42 in ...) {...}' throws SyntaxError in
-    strict mode
+    A direct eval declaring a var named 'arguments' throws SyntaxError
+    in strict mode
 flags: [onlyStrict]
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-    eval('for (var arguments = 42 in null) {};');
+    eval('var arguments;');
 });
