@@ -90,6 +90,8 @@ private Q_SLOTS:
     void initTestCase() override;
 
     void orderedBindings();
+    void scriptBindingValueType_data();
+    void scriptBindingValueType();
     void signalCreationDifferences();
     void allTypesAvailable();
     void shadowing();
@@ -105,6 +107,7 @@ private Q_SLOTS:
     void groupedPropertiesConsistency();
     void groupedPropertySyntax();
     void attachedProperties();
+    void attachedSignalHandler();
     void scriptIndices();
     void extensions();
     void emptyBlockBinding();
@@ -120,6 +123,7 @@ private Q_SLOTS:
     void modulePrefixes();
     void javaScriptBuiltinFlag();
     void isRoot();
+    void idLocation();
 
 public:
     tst_qqmljsscope()
@@ -131,7 +135,7 @@ public:
                           // Note: to be able to import the QQmlJSScopeTests
                           // correctly, we need an additional import path. Use
                           // this application's binary directory as done by
-                          // QQmlImportDatabase
+                          // QQmlTypeLoader
                           QCoreApplication::applicationDirPath(),
                   },
                   nullptr)
@@ -184,6 +188,32 @@ void tst_qqmljsscope::orderedBindings()
     QCOMPARE(std::next(itemsBindingsBegin)->objectType()->baseTypeName(), u"Text"_s);
 }
 
+void tst_qqmljsscope::scriptBindingValueType_data()
+{
+    QTest::addColumn<ScriptBindingValueType>("expectedType");
+
+    QTest::addRow("myInt") << ScriptBindingValueType::ScriptValue_Unknown;
+    QTest::addRow("block") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("alsoABlock") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("lambda") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("namedFunction") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("arrow") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("myUndefined") << ScriptBindingValueType::ScriptValue_Undefined;
+    QTest::addRow("emptyBlock") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("emptyObject") << ScriptBindingValueType::ScriptValue_Unknown;
+}
+void tst_qqmljsscope::scriptBindingValueType()
+{
+    QFETCH(ScriptBindingValueType, expectedType);
+
+    QQmlJSScope::ConstPtr root = run(u"scriptBindingValueType.qml"_s);
+    QVERIFY(root);
+
+    auto [start, end] = root->ownPropertyBindings(QTest::currentDataTag());
+    QCOMPARE(std::distance(start, end), 1);
+    QCOMPARE(start->scriptValueType(), expectedType);
+}
+
 void tst_qqmljsscope::signalCreationDifferences()
 {
     QQmlJSScope::ConstPtr root = run(u"signalCreationDifferences.qml"_s, true);
@@ -454,6 +484,27 @@ void tst_qqmljsscope::attachedProperties()
     QCOMPARE(value(bindingsOfBaseType, u"priority"_s).bindingType(), QQmlSA::BindingType::Script);
 }
 
+void tst_qqmljsscope::attachedSignalHandler()
+{
+    QQmlJSScope::ConstPtr root = run(u"attachedSignalHandler.qml"_s);
+    QVERIFY(root);
+
+    const auto binding = std::find_if(root->childScopesBegin(), root->childScopesEnd(),
+                                      [](const QQmlJSScope::ConstPtr &child) {
+                                          return child->baseTypeName() == "changeHandler"_L1;
+                                      });
+    QCOMPARE_NE(binding, root->childScopesEnd());
+
+    const auto block = std::find_if(
+            (**binding).childScopesBegin(), (**binding).childScopesEnd(),
+            [](const QQmlJSScope::ConstPtr &child) { return child->baseTypeName() == "block"_L1; });
+    QCOMPARE_NE(block, (**binding).childScopesEnd());
+
+    // note: the event JS identifier is inserted only if the "Keys" basetype of the attached
+    // property was resolved before the creation of the "block" scope.
+    QVERIFY((**block).jsIdentifier("event"_L1));
+}
+
 inline QString getScopeName(const QQmlJSScope::ConstPtr &scope)
 {
     Q_ASSERT(scope);
@@ -811,7 +862,7 @@ void tst_qqmljsscope::compilationUnitsAreCompatible()
     QVERIFY2(root, qPrintable(component.errorString()));
     QQmlComponentPrivate *cPriv = QQmlComponentPrivate::get(&component);
     QVERIFY(cPriv);
-    auto unit = cPriv->compilationUnit;
+    auto unit = cPriv->compilationUnit();
     QVERIFY(unit);
     QVERIFY(unit->unitData());
     getRuntimeInfoFromCompilationUnit(unit->unitData(), componentFunctions);
@@ -1062,5 +1113,15 @@ void tst_qqmljsscope::isRoot()
     }
 }
 
+void tst_qqmljsscope::idLocation()
+{
+    auto jsscope = run(u"idSourceLocation.qml"_s, false);
+    const QQmlJS::SourceLocation actual = jsscope->idSourceLocation();
+    QVERIFY(actual.isValid());
+    QCOMPARE(actual.startLine, 7);
+    QCOMPARE(actual.startColumn, 25);
+    QCOMPARE(actual.length, 10);
+}
+
 QTEST_MAIN(tst_qqmljsscope)
 #include "tst_qqmljsscope.moc"
