@@ -1,9 +1,12 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
 #include <qtest.h>
 
 #include <QSignalSpy>
 
+#include <QtCore/QElapsedTimer>
+
 #include <QtQml/QQmlContext>
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlcomponent.h>
@@ -63,6 +66,7 @@ private slots:
     void componentToUrl();
     void anchoredLoader();
     void sizeLoaderToItem();
+    void sizeItemToLoader_data();
     void sizeItemToLoader();
     void noResize();
     void networkRequestUrl();
@@ -108,16 +112,21 @@ private slots:
     void statusChangeOnlyEmittedOnce();
 
     void setSourceAndCheckStatus();
+    void loadComponentWithStates();
     void asyncLoaderRace();
     void noEngine();
 
     void stackOverflow();
+    void stackOverflow2();
+    void boundComponent();
+
+    void invalidateContext();
 };
 
 Q_DECLARE_METATYPE(QList<QQmlError>)
 
 tst_QQuickLoader::tst_QQuickLoader()
-    : QQmlDataTest(QT_QMLTEST_DATADIR)
+    : QQmlDataTest(QT_QMLTEST_DATADIR, FailOnWarningsPolicy::FailOnWarnings)
 {
     qmlRegisterType<SlowComponent>("LoaderTest", 1, 0, "SlowComponent");
     qRegisterMetaType<QList<QQmlError>>();
@@ -304,7 +313,7 @@ void tst_QQuickLoader::componentToUrl()
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
 
-    loader->setSource(testFileUrl("/Rect120x60.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/Rect120x60.qml"));
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
@@ -374,8 +383,29 @@ void tst_QQuickLoader::sizeLoaderToItem()
     QCOMPARE(rect->height(), 30.0);
 }
 
+void tst_QQuickLoader::sizeItemToLoader_data()
+{
+    QTest::addColumn<QString>("property");
+    QTest::addColumn<int>("value");
+    QTest::addColumn<bool>("atOnce");
+
+    QTest::addRow("none_atonce") << "ignore" << 42 << true;
+    QTest::addRow("width_atonce") << "w" << 42 << true;
+    QTest::addRow("height_atonce") << "h" << 42 << true;
+    QTest::addRow("both_atonce") << "both" << 42 << true;
+
+
+    QTest::addRow("none") << "ignore" << 42 << false;
+    QTest::addRow("width") << "w" << 42 << false;
+    QTest::addRow("height") << "h" << 42 << false;
+    QTest::addRow("both") << "both" << 42 << false;
+}
+
 void tst_QQuickLoader::sizeItemToLoader()
 {
+    QFETCH(QString, property);
+    QFETCH(int, value);
+    QFETCH(bool, atOnce);
     QQmlEngine engine;
     QQmlComponent component(&engine, testFileUrl("/SizeToLoader.qml"));
     QScopedPointer<QQuickLoader> loader(qobject_cast<QQuickLoader*>(component.create()));
@@ -385,17 +415,25 @@ void tst_QQuickLoader::sizeItemToLoader()
 
     QQuickItem *rect = qobject_cast<QQuickItem*>(loader->item());
     QVERIFY(rect);
+    rect->setProperty(property.toUtf8(), value);
     QCOMPARE(rect->width(), 200.0);
     QCOMPARE(rect->height(), 80.0);
 
     // Check resize
     QSizeChangeListener sizeListener(rect);
     const QSizeF size(180, 30);
-    loader->setSize(size);
+    if (atOnce) {
+        loader->setSize(size);
+    } else {
+        loader->setWidth(size.width());
+        loader->setHeight(size.height());
+    }
     QVERIFY2(!sizeListener.isEmpty(), "There should be at least one signal about the size changed");
-    for (const QSizeF sizeOnGeometryChanged : sizeListener) {
-        // Check that we have the correct size on all signals
-        QCOMPARE(sizeOnGeometryChanged, size);
+    if (atOnce) {
+        for (const QSizeF sizeOnGeometryChanged : sizeListener) {
+            // Check that we have the correct size on all signals
+            QCOMPARE(sizeOnGeometryChanged, size);
+        }
     }
     QCOMPARE(rect->width(), size.width());
     QCOMPARE(rect->height(), size.height());
@@ -696,13 +734,13 @@ void tst_QQuickLoader::initialPropertyValues_data()
 void tst_QQuickLoader::initialPropertyValues()
 {
     QFETCH(QUrl, qmlFile);
-    QFETCH(QStringList, expectedWarnings);
+    QFETCH(const QStringList, expectedWarnings);
     QFETCH(QStringList, propertyNames);
     QFETCH(QVariantList, propertyValues);
 
     ThreadedTestHTTPServer server(dataDirectory());
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : expectedWarnings)
         QTest::ignoreMessage(QtWarningMsg, QRegularExpression(warning.toLatin1().constData()));
 
     QQmlEngine engine;
@@ -748,8 +786,10 @@ void tst_QQuickLoader::initialPropertyValuesError_data()
     QTest::newRow("nonexistent source url") << testFileUrl("initialPropertyValues.error.2.qml")
             << (QStringList() << QString(testFileUrl("NonexistentSourceComponent.qml").toString() + ": No such file or directory"));
 
-    QTest::newRow("invalid source url") << testFileUrl("initialPropertyValues.error.3.qml")
-            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString() + ":5:1: Expected token `:'"));
+    QTest::newRow("invalid source url")
+            << testFileUrl("initialPropertyValues.error.3.qml")
+            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString()
+                                         + ":4:5: Incomplete binding, expected token `:` or `{`"));
 
     QTest::newRow("invalid initial property values object with invalid property access") << testFileUrl("initialPropertyValues.error.4.qml")
             << (QStringList() << QString(testFileUrl("initialPropertyValues.error.4.qml").toString() + ":7:5: QML Loader: setSource: value is not an object")
@@ -759,9 +799,9 @@ void tst_QQuickLoader::initialPropertyValuesError_data()
 void tst_QQuickLoader::initialPropertyValuesError()
 {
     QFETCH(QUrl, qmlFile);
-    QFETCH(QStringList, expectedWarnings);
+    QFETCH(const QStringList, expectedWarnings);
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : expectedWarnings)
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QQmlEngine engine;
@@ -894,19 +934,21 @@ void tst_QQuickLoader::asynchronous_data()
     QTest::newRow("Non-existent component") << testFileUrl("IDoNotExist.qml")
             << (QStringList() << QString(testFileUrl("IDoNotExist.qml").toString() + ": No such file or directory"));
 
-    QTest::newRow("Invalid component") << testFileUrl("InvalidSourceComponent.qml")
-            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString() + ":5:1: Expected token `:'"));
+    QTest::newRow("Invalid component")
+            << testFileUrl("InvalidSourceComponent.qml")
+            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString()
+                                         + ":4:5: Incomplete binding, expected token `:` or `{`"));
 }
 
 void tst_QQuickLoader::asynchronous()
 {
     QFETCH(QUrl, qmlFile);
-    QFETCH(QStringList, expectedWarnings);
+    QFETCH(const QStringList, expectedWarnings);
 
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -916,7 +958,7 @@ void tst_QQuickLoader::asynchronous()
     QQuickLoader *loader = root->findChild<QQuickLoader*>("loader");
     QVERIFY(loader);
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : expectedWarnings)
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QVERIFY(!loader->item());
@@ -943,9 +985,9 @@ void tst_QQuickLoader::asynchronous()
 void tst_QQuickLoader::asynchronous_clear()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -991,9 +1033,9 @@ void tst_QQuickLoader::asynchronous_clear()
 void tst_QQuickLoader::simultaneousSyncAsync()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("simultaneous.qml"));
@@ -1023,9 +1065,9 @@ void tst_QQuickLoader::simultaneousSyncAsync()
 void tst_QQuickLoader::asyncToSync1()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -1055,9 +1097,9 @@ void tst_QQuickLoader::asyncToSync1()
 void tst_QQuickLoader::asyncToSync2()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -1087,9 +1129,9 @@ void tst_QQuickLoader::asyncToSync2()
 void tst_QQuickLoader::loadedSignal()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     {
@@ -1421,7 +1463,7 @@ void tst_QQuickLoader::sourceURLKeepComponent()
                       dataDirectoryUrl());
 
     QScopedPointer<QQuickLoader> loader(qobject_cast<QQuickLoader*>(component.create()));
-    loader->setSource(testFileUrl("/Rect120x60.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/Rect120x60.qml"));
 
     QVERIFY(loader);
     QVERIFY(loader->item());
@@ -1447,7 +1489,7 @@ void tst_QQuickLoader::sourceURLKeepComponent()
     QCOMPARE(sourceComponent.data(), loader->sourceComponent());
 
     //Ensure changing source url causes component to be recreated when inactive
-    loader->setSource(testFileUrl("/BlueRect.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/BlueRect.qml"));
 
     loader->setActive(true);
     QVERIFY(loader->item());
@@ -1457,7 +1499,7 @@ void tst_QQuickLoader::sourceURLKeepComponent()
     QVERIFY(sourceComponent.data() != newSourceComponent.data());
 
     //Ensure changing source url causes component to be recreated when active
-    loader->setSource(testFileUrl("/Rect120x60.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/Rect120x60.qml"));
     QVERIFY(loader->sourceComponent() != newSourceComponent.data());
 
 }
@@ -1489,10 +1531,30 @@ void tst_QQuickLoader::setSourceAndCheckStatus()
     QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant::fromValue(QStringLiteral(""))));
     QCOMPARE(loader->status(), QQuickLoader::Null);
 
-    QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant()));
+    QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant(QUrl())));
     QCOMPARE(loader->status(), QQuickLoader::Null);
 }
 
+void tst_QQuickLoader::loadComponentWithStates()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData(QByteArray("import QtQuick\n"
+                                 "Loader {\n"
+                                     "id: loader\n"
+                                     "property int createdObjCount: 0\n"
+                                     "states: [ State { when: true; PropertyChanges { target: loader; sourceComponent: myComp } } ]\n"
+                                     "Component { id: myComp; Item { Component.onCompleted: { ++createdObjCount } } }\n"
+                                 "}" )
+            , dataDirectoryUrl());
+    QScopedPointer<QQuickLoader> loader(qobject_cast<QQuickLoader*>(component.create()));
+    QTest::qWait(200);
+    QTRY_VERIFY(loader != nullptr);
+    QVERIFY(loader->item());
+    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), 1);
+    QCOMPARE(loader->property("createdObjCount").toInt(), 1);
+}
+
 void tst_QQuickLoader::asyncLoaderRace()
 {
     QQmlApplicationEngine engine;
@@ -1525,10 +1587,9 @@ void tst_QQuickLoader::noEngine()
     QTRY_COMPARE(o->property("changes").toInt(), 1);
 }
 
-void tst_QQuickLoader::stackOverflow()
+static void qTestForOverflow(const QUrl &url)
 {
     QQmlEngine engine;
-    const QUrl url = testFileUrl("overflow.qml");
     QQmlComponent component(&engine, url);
     QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     const QString message = url.toString() + QStringLiteral(": Maximum call stack size exceeded.");
@@ -1536,6 +1597,85 @@ void tst_QQuickLoader::stackOverflow()
     QScopedPointer<QObject> o(component.create());
 }
 
+void tst_QQuickLoader::stackOverflow()
+{
+    auto t = QThread::create(qTestForOverflow, testFileUrl("overflow.qml"));
+    t->setStackSize(1024 * 1024);
+    t->start();
+    t->wait();
+}
+
+void tst_QQuickLoader::stackOverflow2()
+{
+    auto t = QThread::create(qTestForOverflow, testFileUrl("overflow2.qml"));
+    t->setStackSize(1024 * 1024);
+    t->start();
+    t->wait();
+}
+
+void tst_QQuickLoader::boundComponent()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("boundComponent.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> o(component.create());
+    QCOMPARE(o->objectName(), QStringLiteral("loaded"));
+}
+
+class CppModel : public QAbstractListModel
+{
+public:
+    virtual int rowCount(const QModelIndex &) const override {return 1;}
+    virtual QVariant data(const QModelIndex &, int) const override { return "test"; }
+
+    void reset () {beginResetModel(); endResetModel();}
+};
+
+class RootData : public QObject
+{
+    Q_OBJECT
+public:
+    RootData() : object(std::make_unique<QObject>())
+    {
+        object->setObjectName("objectName");
+    }
+
+    Q_INVOKABLE QString getValue() const {  return object->objectName(); }
+
+    void deleteObject() { object.reset(); }
+
+private:
+    std::unique_ptr<QObject> object;
+};
+
+void tst_QQuickLoader::invalidateContext()
+{
+    CppModel model;
+    RootData rootData;
+
+    QQmlEngine engine;
+    QQmlContext *rootContext = engine.rootContext();
+    rootContext->setContextProperty("cppModel", &model);
+    rootContext->setContextProperty("loaderActive", true);
+    rootContext->setContextProperty("rootData", &rootData);
+
+    QQmlComponent component(&engine, testFileUrl("invalidateContext.qml"));\
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+
+    QTest::ignoreMessage(QtDebugMsg, "Repeater constructed");
+    QTest::ignoreMessage(QtDebugMsg, "updating text");
+    QScopedPointer<QObject> o(component.create());
+
+    QTest::ignoreMessage(QtDebugMsg, "Repeater destroyed");
+    rootContext->setContextProperty("loaderActive", false);
+
+    // The object should never be used anymore, as loader is inactive
+    rootData.deleteObject();
+
+    // Should not trigger any re-evaluation (which would crash in getValue())
+    model.reset();
+}
+
 QTEST_MAIN(tst_QQuickLoader)
 
 #include "tst_qquickloader.moc"
