@@ -4,7 +4,8 @@
 #include <QtGui/qpa/qplatformintegration.h>
 #include <QtGui/qpa/qplatformtheme.h>
 #include <QtGui/private/qguiapplication_p.h>
-#include <QtTest>
+#include <QTest>
+#include <QSignalSpy>
 #include <QtQml>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
@@ -17,6 +18,7 @@
 #include <QtQuickTemplates2/private/qquickmenubaritem_p.h>
 #include <QtQuickTemplates2/private/qquickmenuitem_p.h>
 #include <QtQuickTemplates2/private/qquickpopupwindow_p_p.h>
+#include <QtQuickTest/quicktest.h>
 #include <QtQuickControlsTestUtils/private/controlstestutils_p.h>
 #include <QtQuickControlsTestUtils/private/qtest_quickcontrols_p.h>
 
@@ -79,10 +81,11 @@ private slots:
     void invalidDelegate();
     void panMenuBar_data();
     void panMenuBar();
+    void clearMenus();
+    void popupWindowsKeyboardNavigation();
 
 private:
     bool nativeMenuBarSupported = false;
-    bool popupWindowsSupported = false;
     QScopedPointer<QPointingDevice> touchScreen = QScopedPointer<QPointingDevice>(QTest::createTouchDevice());
 };
 
@@ -97,9 +100,6 @@ tst_qquickmenubar::tst_qquickmenubar()
     qputenv("QML_NO_TOUCH_COMPRESSION", "1");
     QQuickMenuBar mb;
     nativeMenuBarSupported = QQuickMenuBarPrivate::get(&mb)->useNativeMenuBar();
-#if defined(Q_OS_WINDOWS) || defined(Q_OS_MACOS)
-    popupWindowsSupported = QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::Capability::MultipleWindows);
-#endif
 }
 
 void tst_qquickmenubar::init()
@@ -137,7 +137,7 @@ void tst_qquickmenubar::mouse_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -207,7 +207,7 @@ void tst_qquickmenubar::mouse()
     // re-trigger a menubar item to hide the menu - it should close on press
     QTest::mousePress(window.data(), Qt::LeftButton, Qt::NoModifier, itemSceneCenter(editMenuBarItem));
     QVERIFY(editMenuBarItem->isHighlighted());
-    QVERIFY(editMenuBarItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarItem);
     QTRY_VERIFY(!editMenuBarMenu->isVisible());
     QTest::mouseRelease(window.data(), Qt::LeftButton, Qt::NoModifier, itemSceneCenter(editMenuBarItem));
 
@@ -363,7 +363,7 @@ void tst_qquickmenubar::keys_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -373,7 +373,7 @@ void tst_qquickmenubar::keys()
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuWindows);
 
-    SKIP_IF_NO_WINDOW_ACTIVATION
+    SKIP_IF_NO_WINDOW_ACTIVATION;
 
     QQmlApplicationEngine engine(testFileUrl("menubaritems.qml"));
 
@@ -408,7 +408,7 @@ void tst_qquickmenubar::keys()
     QVERIFY(editMenuBarItem->isHighlighted());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
 
     // navigate down to the menu
     QQuickMenuItem *cutMenuItem = qobject_cast<QQuickMenuItem *>(editMenuBarMenu->itemAt(0));
@@ -417,13 +417,13 @@ void tst_qquickmenubar::keys()
     QVERIFY(!cutMenuItem->hasActiveFocus());
     QTest::keyClick(window.data(), Qt::Key_Down);
     QVERIFY(cutMenuItem->isHighlighted());
-    QVERIFY(cutMenuItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(cutMenuItem);
 
     // navigate up, back to the menubar
     QTest::keyClick(window.data(), Qt::Key_Up);
     QTRY_VERIFY(!editMenuBarMenu->isVisible());
     QVERIFY(editMenuBarItem->isHighlighted());
-    QVERIFY(editMenuBarItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarItem);
 
 // There seem to be problems in focus handling in webOS QPA, see https://bugreports.qt.io/browse/WEBOSCI-45
 #ifdef Q_OS_WEBOS
@@ -438,9 +438,9 @@ void tst_qquickmenubar::keys()
     QVERIFY(!editMenuBarItem->hasActiveFocus());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
     QVERIFY(cutMenuItem->isHighlighted());
-    QVERIFY(cutMenuItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(cutMenuItem);
 
     // navigate left in popup mode (menu open)
     QTest::keyClick(window.data(), Qt::Key_Left);
@@ -484,7 +484,7 @@ void tst_qquickmenubar::keys()
     QVERIFY(viewMenuBarMenu->isVisible());
     QTRY_VERIFY(viewMenuBarMenu->isOpened());
     QVERIFY(!viewMenuBarItem->hasActiveFocus());
-    QVERIFY(viewMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(viewMenuBarMenu);
 
     // open a sub-menu
     QQuickMenuItem *alignmentSubMenuItem = qobject_cast<QQuickMenuItem *>(viewMenuBarMenu->itemAt(0));
@@ -570,7 +570,7 @@ void tst_qquickmenubar::mnemonics_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -580,7 +580,7 @@ void tst_qquickmenubar::mnemonics()
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuWindows);
 
-    SKIP_IF_NO_WINDOW_ACTIVATION
+    SKIP_IF_NO_WINDOW_ACTIVATION;
 
 #if defined(Q_OS_MACOS) or defined(Q_OS_WEBOS)
     QSKIP("Mnemonics are not used on this platform");
@@ -629,7 +629,7 @@ void tst_qquickmenubar::mnemonics()
     QVERIFY(!editMenuBarItem->hasActiveFocus());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
 
     // press Alt to hide the menu
     keySim.click(Qt::Key_Alt);
@@ -645,7 +645,7 @@ void tst_qquickmenubar::mnemonics()
     QVERIFY(editMenuBarItem->isHighlighted());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
     QVERIFY(!editMenuBarItem->hasActiveFocus());
 
     // trigger another menubar item to open another menu, leave Alt pressed
@@ -715,7 +715,7 @@ void tst_qquickmenubar::mnemonics()
     QVERIFY(fileMenuBarItem->isHighlighted());
     QVERIFY(fileMenuBarMenu->isVisible());
     QTRY_VERIFY(fileMenuBarMenu->isOpened());
-    QVERIFY(fileMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(fileMenuBarMenu);
 
     // trigger a menu item to close the menu, which shouldn't trigger a button
     // action behind the menu (QTBUG-86276)
@@ -761,7 +761,7 @@ void tst_qquickmenubar::altNavigation()
     // releasing Alt with AltModifier removes the AltModifier.
     QTest::keyPress(window.get(), Qt::Key_Alt);
     QTest::keyRelease(window.get(), Qt::Key_Alt, Qt::AltModifier);
-    QVERIFY(menuBar->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(menuBar);
     QVERIFY(fileMenuBarItem->isHighlighted());
 
     // if menu has focus, pressing the mnemonic without Alt should open the menu
@@ -772,7 +772,7 @@ void tst_qquickmenubar::altNavigation()
     QVERIFY(editMenuBarItem->isHighlighted());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
 }
 
 void tst_qquickmenubar::addRemove_data()
@@ -1098,7 +1098,7 @@ void tst_qquickmenubar::addRemoveExistingMenus_data()
     if (nativeMenuBarSupported)
         QTest::newRow("native, native") << true << true;
     // Uncomment when popup windows work 100%
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("non-native, popup window") << false << true;
 }
 
@@ -1135,7 +1135,7 @@ void tst_qquickmenubar::checkHighlightWhenMenuDismissed_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -1207,7 +1207,7 @@ void tst_qquickmenubar::hoverAfterClosingWithEscape_data()
     QTest::addColumn<bool>("usePopupWindow");
     QTest::newRow("in-scene popup") << false;
     // Uncomment when popup windows work 100%
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("popup window") << true;
 }
 
@@ -1306,14 +1306,15 @@ void tst_qquickmenubar::AA_DontUseNativeMenuBar()
     QQuickMenuBar *menuBar = window->property("menuBar").value<QQuickMenuBar *>();
     QVERIFY(menuBar);
     auto menuBarPrivate = QQuickMenuBarPrivate::get(menuBar);
-    QQuickItem *contents = window->property("contents").value<QQuickItem *>();
+    QQuickItem *contents = window->contentItem();
     QVERIFY(contents);
 
     QVERIFY(!menuBarPrivate->nativeHandle());
     QVERIFY(menuBar->isVisible());
     QVERIFY(menuBar->count() > 0);
     QVERIFY(menuBar->height() > 0);
-    QCOMPARE(contents->height(), window->height() - menuBar->height());
+    auto bottomSafeMargin = window->safeAreaMargins().bottom();
+    QCOMPARE(contents->height(), window->height() - menuBar->height() - bottomSafeMargin);
 
     // If the menu bar is not native, the menus should not be native either.
     // The main reason for this limitation is that a native menu typically
@@ -1510,9 +1511,14 @@ void tst_qquickmenubar::applicationWindow()
     QQuickMenuBar *menuBar = window->property("menuBar").value<QQuickMenuBar *>();
     QVERIFY(menuBar);
     auto menuBarPrivate = QQuickMenuBarPrivate::get(menuBar);
-    QQuickItem *contents = window->property("contents").value<QQuickItem *>();
+    QQuickItem *contents = window->contentItem();
     QVERIFY(contents);
 
+    // The window may report safe area margins when invisible, but they will not
+    // propagate to the Quick SafeArea until shown.
+    auto topSafeMargin = initiallyVisible ? window->safeAreaMargins().top() : 0;
+    auto bottomSafeMargin = initiallyVisible ? window->safeAreaMargins().bottom() : 0;
+
     for (const bool visible : {initiallyVisible, !initiallyVisible, initiallyVisible}) {
         menuBar->setVisible(visible);
 
@@ -1522,14 +1528,14 @@ void tst_qquickmenubar::applicationWindow()
         if (!visible) {
             QVERIFY(!menuBar->isVisible());
             QVERIFY(!nativeMenuBarVisible);
-            QCOMPARE(contents->height(), window->height());
+            QCOMPARE(contents->height(), window->height() - topSafeMargin - bottomSafeMargin);
         } else if (nativeMenuBarVisible) {
             QVERIFY(menuBar->isVisible());
-            QCOMPARE(contents->height(), window->height());
+            QCOMPARE(contents->height(), window->height() - topSafeMargin - bottomSafeMargin);
         } else {
             QVERIFY(menuBar->isVisible());
             QVERIFY(menuBar->height() > 0);
-            QCOMPARE(contents->height(), window->height() - menuBar->height());
+            QCOMPARE(contents->height(), window->height() - menuBar->height() - bottomSafeMargin);
         }
     }
 }
@@ -1558,17 +1564,18 @@ void tst_qquickmenubar::menubarAsHeader()
     QQuickMenuBar *menuBar = window->property("header").value<QQuickMenuBar *>();
     QVERIFY(menuBar);
     auto menuBarPrivate = QQuickMenuBarPrivate::get(menuBar);
-    QQuickItem *contents = window->property("contents").value<QQuickItem *>();
+    QQuickItem *contents = window->contentItem();
     QVERIFY(contents);
     QVERIFY(menuBar->count() > 0);
     QCOMPARE(menuBarPrivate->nativeHandle() != nullptr, native);
 
     if (menuBarPrivate->nativeHandle()) {
         // Using native menubar
-        QCOMPARE(contents->height(), window->height());
+        QCOMPARE(contents->height(), window->height() - window->safeAreaMargins().top());
     } else {
         // Not using native menubar
-        QCOMPARE(contents->height(), window->height() - menuBar->height());
+        auto bottomSafeMargin = window->safeAreaMargins().bottom();
+        QCOMPARE(contents->height(), window->height() - menuBar->height() - bottomSafeMargin);
     }
 }
 
@@ -1576,7 +1583,7 @@ void tst_qquickmenubar::menuPosition_data()
 {
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
-    if (popupWindowsSupported)
+    if (arePopupWindowsSupported())
         QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -1815,7 +1822,7 @@ void tst_qquickmenubar::panMenuBar_data()
     QTest::addColumn<bool>("usePopupWindow");
     QTest::newRow("in-scene popup") << false;
     // Uncomment when popup windows work 100%
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("popup window") << true;
 }
 
@@ -1866,6 +1873,142 @@ void tst_qquickmenubar::panMenuBar()
     QTRY_VERIFY(!menuBarItem0->menu()->isOpened());
 }
 
+void tst_qquickmenubar::clearMenus()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("clearMenus.qml"));
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QTRY_COMPARE(o->property("v").toInt(), 2);
+}
+
+void tst_qquickmenubar::popupWindowsKeyboardNavigation()
+{
+    if (!arePopupWindowsSupported())
+        QSKIP("The platform doesn't support popup windows. Skipping test.");
+    QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar, true);
+    QQuickApplicationHelper helper(this, "popupWindowMenus.qml");
+    QVERIFY2(helper.ready, helper.failureMessage());
+    QQuickApplicationWindow *window = qobject_cast<QQuickApplicationWindow *>(helper.window);
+    QVERIFY(window);
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+    QQuickMenuBar *menuBar = window->property("menuBar").value<QQuickMenuBar *>();
+    QVERIFY(menuBar);
+    QTRY_VERIFY(menuBar->hasActiveFocus());
+    QQuickMenuBarItem *menuBarItem1 = qobject_cast<QQuickMenuBarItem *>(menuBar->itemAt(0));
+    QVERIFY(menuBarItem1);
+    QVERIFY(!menuBarItem1->hasActiveFocus());
+    QTest::keyClick(window, Qt::Key_Right, Qt::NoModifier);
+    QTRY_VERIFY(menuBarItem1->isHighlighted());
+    QVERIFY(QQuickTest::qWaitForPolish(window));
+    QGuiApplication::processEvents();
+
+    QQuickMenu *menu1 = menuBar->menuAt(0);
+    QVERIFY(menu1);
+    const auto menu1Private = QQuickMenuPrivate::get(menu1);
+
+    // Open the "File" menu
+    QTest::keyClick(window, Qt::Key_Down, Qt::NoModifier);
+    QTRY_VERIFY(menu1Private->popupWindow);
+    QCOMPARE(menu1Private->popupWindow->transientParent(), window);
+    QTRY_VERIFY(menu1->isOpened());
+    QVERIFY(QTest::qWaitForWindowExposed(menu1Private->popupWindow));
+    QTRY_VERIFY(!menu1Private->transitionManager.isRunning());
+    QVERIFY(QQuickTest::qWaitForPolish(menu1Private->popupWindow));
+
+    const QQuickMenuItem *menuItem1 = qobject_cast<QQuickMenuItem *>(menu1->itemAt(0));
+    QVERIFY(menuItem1);
+
+    // Popup windows don't become focus windows, so the focus window should still be the ApplicationWindow.
+    // TODO: On wayland, the popup window will sometimes becomes the focus window. Why?
+    if (!QGuiApplication::platformName().startsWith(QStringLiteral("wayland")))
+        QTRY_COMPARE(QGuiApplication::focusWindow(), window);
+    QTRY_VERIFY_ACTIVE_FOCUS(menuItem1);
+
+    QTest::keyClick(window, Qt::Key_Down, Qt::NoModifier);
+
+    const QQuickMenuItem *menuItem2 = qobject_cast<QQuickMenuItem *>(menu1->itemAt(1));
+
+    QVERIFY(menuItem2);
+    QTRY_VERIFY_ACTIVE_FOCUS(menuItem2);
+
+    QTest::keyClick(window, Qt::Key_Down, Qt::NoModifier);
+
+    const QQuickItem *menuItem3 = qobject_cast<QQuickItem *>(menu1->itemAt(2));
+    QVERIFY(menuItem3);
+    QTRY_VERIFY_ACTIVE_FOCUS(menuItem3);
+
+    QTest::keyClick(window, Qt::Key_Right, Qt::NoModifier);
+
+    // Open the submenu
+    QQuickMenu *subMenu = window->property("subMenu").value<QQuickMenu *>();
+    QVERIFY(subMenu);
+    const auto *subMenuPrivate = QQuickMenuPrivate::get(subMenu);
+    QTRY_VERIFY(subMenuPrivate->popupWindow);
+    QCOMPARE(subMenuPrivate->popupWindow->transientParent(), menu1Private->popupWindow);
+    QTRY_VERIFY(subMenu->isOpened());
+    QVERIFY(QTest::qWaitForWindowExposed(subMenuPrivate->popupWindow));
+    QTRY_VERIFY(!subMenuPrivate->transitionManager.isRunning());
+    QVERIFY(QQuickTest::qWaitForPolish(subMenuPrivate->popupWindow));
+
+    const auto *subMenuItem1 = subMenu->itemAt(0);
+    QVERIFY(subMenuItem1);
+    QTRY_VERIFY_ACTIVE_FOCUS(subMenuItem1);
+    QGuiApplication::processEvents();
+
+    QTest::keyClick(window, Qt::Key_Down, Qt::NoModifier);
+    const auto *subMenuItem2 = subMenu->itemAt(1);
+    QVERIFY(subMenuItem2);
+    QTRY_VERIFY_ACTIVE_FOCUS(subMenuItem2);
+    QGuiApplication::processEvents();
+
+    // Close submenu
+    QTest::keyClick(window, Qt::Key_Escape, Qt::NoModifier);
+    QTRY_VERIFY(!subMenu->isVisible());
+    QTRY_VERIFY_ACTIVE_FOCUS(menuItem3);
+    QGuiApplication::processEvents();
+
+    // Move back to the menubar
+    QTest::keyClick(window, Qt::Key_Up, Qt::NoModifier);
+    QTRY_VERIFY_ACTIVE_FOCUS(menuItem2);
+
+    QTest::keyClick(window, Qt::Key_Up, Qt::NoModifier);
+    QTRY_VERIFY_ACTIVE_FOCUS(menuItem1);
+
+    QTest::keyClick(window, Qt::Key_Up, Qt::NoModifier);
+    QTRY_VERIFY(!menu1->isVisible());
+    QTRY_VERIFY_ACTIVE_FOCUS(menuBar);
+    QCOMPARE(window->activeFocusItem(), menuBar);
+    QVERIFY(menuBarItem1->isHighlighted());
+
+    QTest::keyClick(window, Qt::Key_Right, Qt::NoModifier);
+    QQuickMenuBarItem *menuBarItem2 = qobject_cast<QQuickMenuBarItem *>(menuBar->itemAt(1));
+    QVERIFY(menuBarItem2);
+    QTRY_VERIFY(menuBarItem2->isHighlighted());
+
+    // Open the "Edit" menu
+    QTest::keyClick(window, Qt::Key_Down, Qt::NoModifier);
+    QQuickMenu *menu2 = menuBarItem2->menu();
+    QVERIFY(menu2);
+    const auto menu2Private = QQuickMenuPrivate::get(menu2);
+    QTRY_VERIFY(menu2Private->popupWindow);
+    QCOMPARE(menu2Private->popupWindow->transientParent(), window);
+    QTRY_VERIFY(menu2->isOpened());
+    QVERIFY(QTest::qWaitForWindowExposed(menu2Private->popupWindow));
+    QTRY_VERIFY(!menu2Private->transitionManager.isRunning());
+    QVERIFY(QQuickTest::qWaitForPolish(menu2Private->popupWindow));
+
+    const auto *menu2item1 = qobject_cast<QQuickItem *>(menu2->itemAt(0));
+    QTRY_VERIFY_ACTIVE_FOCUS(menu2item1);
+    QTest::keyClick(window, Qt::Key_Enter, Qt::NoModifier);
+    QTRY_VERIFY(!menu2->isVisible());
+    QTRY_VERIFY_ACTIVE_FOCUS(menuBar);
+    QVERIFY(!menuBarItem2->isHighlighted());
+}
+
+
 QTEST_QUICKCONTROLS_MAIN(tst_qquickmenubar)
 
 #include "tst_qquickmenubar.moc"
