@@ -3,6 +3,7 @@
 
 #include <QtTest/qtest.h>
 #include <QtTest/qsignalspy.h>
+#include <QtTest/private/qtesthelpers_p.h>
 
 #include <QtQuick/qquickwindow.h>
 #include <QtQuick/qquickitem.h>
@@ -28,6 +29,7 @@ private slots:
     void clearGrabbers();
     void retainOrientation();
     void pressedAndReleased();
+    void eventsToWindowlessPopupInAsyncLoader();
 };
 
 class TestInputHandler : public QQuickPointerDeviceHandler
@@ -237,6 +239,9 @@ void tst_QQuickOverlay::pressedAndReleased()
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
     QVERIFY(QTest::qWaitForWindowExposed(window));
+    // Verify that the position and framePosition are correct
+    // to avoid flakiness on openSuSe KDE.
+    QVERIFY(QTestPrivate::ensurePositionTopLeft(window));
 
     auto *overlay = QQuickOverlay::overlay(window);
     QVERIFY(overlay);
@@ -267,6 +272,33 @@ void tst_QQuickOverlay::pressedAndReleased()
     QCOMPARE(tappedSpy.count(), 1);
 }
 
+void tst_QQuickOverlay::eventsToWindowlessPopupInAsyncLoader()
+{
+    QQuickControlsApplicationHelper helper(this, "eventsToWindowlessPopupInAsyncLoader.qml");
+    QVERIFY2(helper.ready, helper.failureMessage());
+    QQuickApplicationWindow *window = helper.appWindow;
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+
+    const QPoint startPos(0, 0);
+    const QPoint targetPos(40, 40);
+    const int steps = 80;
+    const int delay = 1;
+
+    // Test various event types. None of them should cause crashes.
+    PointLerper mousePointLerper(window, startPos);
+    mousePointLerper.move(targetPos, steps, delay);
+
+    std::unique_ptr<QPointingDevice> touchscreen { QTest::createTouchDevice() };
+    PointLerper touchPointLerper(window, startPos, touchscreen.get());
+    touchPointLerper.move(targetPos, steps, delay);
+
+    forEachStep(steps, [&](qreal progress) {
+        QTest::mouseClick(window, Qt::LeftButton, Qt::NoModifier,
+            lerpPoints(startPos, targetPos, progress), delay);
+    });
+}
+
 QTEST_MAIN(tst_QQuickOverlay)
 
 #include "tst_qquickoverlay.moc"
