@@ -45,8 +45,10 @@ static bool isExplicitComponent(const QQmlJSScope::ConstPtr &type)
  */
 static bool isImplicitComponent(const QQmlJSScope::ConstPtr &type)
 {
-    if (!type->isComposite())
+    // root components and inline components are explicitly components.
+    if (!type->isComposite() || type->isFileRootComponent() || type->isInlineComponent())
         return false;
+
     const auto cppBase = QQmlJSScope::nonCompositeBaseType(type);
     const bool isComponentBased = (cppBase && cppBase->internalName() == u"QQmlComponent");
     return type->componentRootStatus() != QQmlJSScope::IsComponentRoot::No && !isComponentBased;
@@ -337,6 +339,13 @@ void QmltcVisitor::endVisit(QQmlJS::AST::UiProgram *program)
             checkNamesAndTypes(type);
 }
 
+bool QmltcVisitor::checkCustomParser(const QQmlJSScope::ConstPtr &scope)
+{
+    if (QQmlJSImportVisitor::checkCustomParser(scope))
+        m_seenCustomParsers = true;
+    return false;
+}
+
 QQmlJSScope::ConstPtr fetchType(const QQmlJSMetaPropertyBinding &binding)
 {
     switch (binding.bindingType()) {
@@ -347,7 +356,7 @@ QQmlJSScope::ConstPtr fetchType(const QQmlJSMetaPropertyBinding &binding)
     case QQmlSA::BindingType::ValueSource:
         return binding.valueSourceType();
     case QQmlSA::BindingType::AttachedProperty:
-        return binding.attachingType();
+        return binding.attachedType();
     case QQmlSA::BindingType::GroupProperty:
         return binding.groupType();
     default:
@@ -356,11 +365,11 @@ QQmlJSScope::ConstPtr fetchType(const QQmlJSMetaPropertyBinding &binding)
     Q_UNREACHABLE_RETURN({});
 }
 
-template<typename Predicate>
-void iterateTypes(
+template<typename TypePredicate, typename BindingPredicate>
+void iterateBindings(
         const QQmlJSScope::ConstPtr &root,
         const QHash<QQmlJSScope::ConstPtr, QList<QQmlJSMetaPropertyBinding>> &qmlIrOrderedBindings,
-        Predicate predicate)
+        TypePredicate typePredicate, BindingPredicate bindingPredicate)
 {
     // NB: depth-first-search is used here to mimic various QmlIR passes
     QStack<QQmlJSScope::ConstPtr> types;
@@ -368,10 +377,7 @@ void iterateTypes(
     while (!types.isEmpty()) {
         auto current = types.pop();
 
-        if (predicate(current))
-            continue;
-
-        if (isOrUnderComponent(current)) // ignore implicit/explicit components
+        if (typePredicate(current))
             continue;
 
         Q_ASSERT(qmlIrOrderedBindings.contains(current));
@@ -380,6 +386,10 @@ void iterateTypes(
         // child first and we need left-most first
         for (auto it = bindings.rbegin(); it != bindings.rend(); ++it) {
             const auto &binding = *it;
+
+            if (bindingPredicate(current, binding))
+                continue;
+
             if (auto type = fetchType(binding))
                 types.push(type);
         }
@@ -387,34 +397,16 @@ void iterateTypes(
 }
 
 template<typename Predicate>
-void iterateBindings(
+void iterateTypes(
         const QQmlJSScope::ConstPtr &root,
         const QHash<QQmlJSScope::ConstPtr, QList<QQmlJSMetaPropertyBinding>> &qmlIrOrderedBindings,
         Predicate predicate)
 {
-    // NB: depth-first-search is used here to mimic various QmlIR passes
-    QStack<QQmlJSScope::ConstPtr> types;
-    types.push(root);
-    while (!types.isEmpty()) {
-        auto current = types.pop();
-
-        if (isOrUnderComponent(current)) // ignore implicit/explicit components
-            continue;
-
-        Q_ASSERT(qmlIrOrderedBindings.contains(current));
-        const auto &bindings = qmlIrOrderedBindings[current];
-        // reverse the binding order here, because stack processes right-most
-        // child first and we need left-most first
-        for (auto it = bindings.rbegin(); it != bindings.rend(); ++it) {
-            const auto &binding = *it;
-
-            if (predicate(current, binding))
-                continue;
-
-            if (auto type = fetchType(binding))
-                types.push(type);
-        }
-    }
+    iterateBindings(root, qmlIrOrderedBindings, [predicate](const QQmlJSScope::ConstPtr &current) {
+        return predicate(current) || isOrUnderComponent(current);
+    }, [](const QQmlJSScope::ConstPtr &, const QQmlJSMetaPropertyBinding &) {
+        return false;
+    });
 }
 
 /*! \internal
@@ -462,7 +454,7 @@ void QmltcVisitor::postVisitResolve(
     };
     for (const auto &inlineComponentName : m_inlineComponentNames) {
         iterateBindings(m_inlineComponents[inlineComponentName], qmlIrOrderedBindings,
-                        findDeferred);
+                        isOrUnderComponent, findDeferred);
     }
 
     const auto isOrUnderDeferred = [&deferredTypes](QQmlJSScope::ConstPtr type) {
@@ -590,7 +582,13 @@ void QmltcVisitor::postVisitResolve(
     const auto setRuntimeId = [&](const QQmlJSScope::ConstPtr &type) {
         // any type wrapped in an implicit component shouldn't be processed
         // here. even if it has id, it doesn't need to be set by qmltc
-        if (type->componentRootStatus() != QQmlJSScope::IsComponentRoot::No) {
+        if (type->isInlineComponent()) {
+            // explicit inline component
+        } else if (type->isFileRootComponent()) {
+            // explicit root component
+        } else if (type->componentRootStatus() != QQmlJSScope::IsComponentRoot::No) {
+            // Wrapped in implicit component, assigned to unknown property, or child of scope
+            // called "QQmlComponent". We consider this an "implicit component".
             return true;
         }
 
@@ -773,7 +771,16 @@ void QmltcVisitor::checkNamesAndTypes(const QQmlJSScope::ConstPtr &type)
 
     const auto validateType = [&type, this](const QQmlJSScope::ConstPtr &typeToCheck,
                                             QStringView name, QStringView errorPrefix) {
-        if (type->moduleName().isEmpty() || typeToCheck.isNull())
+        if (typeToCheck.isNull()) {
+            m_logger->log(
+                    QStringLiteral(
+                            "Can't compile the %1 type \"%2\" to C++ because it cannot be resolved")
+                            .arg(errorPrefix, name),
+                    qmlCompiler, type->sourceLocation());
+            return;
+        }
+
+        if (type->moduleName().isEmpty())
             return;
 
         if (typeToCheck->isComposite() && typeToCheck->moduleName() != type->moduleName()) {
